//
//  CoreDataSingleton.h
//  EducationPlus
//
//  Created by Jon Freberg on 3/23/12.
//  Copyright (c) 2012 FrebergWeb. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "User+Create.h"

@interface CoreDataSingleton : NSObject {
    NSManagedObjectModel *managedObjectModel;
    NSManagedObjectContext *managedObjectContext;
    NSPersistentStoreCoordinator *persistentStoreCoordinator;
}

@property (nonatomic, retain, readonly) NSManagedObjectModel *managedObjectModel;
@property (nonatomic, retain, readonly) NSManagedObjectContext *managedObjectContext;
@property (nonatomic, retain, readonly) NSPersistentStoreCoordinator *persistentStoreCoordinator;
@property (nonatomic, readonly) NSString *applicationDocumentsDirectory;

// Instance methods
- (NSMutableArray*)fetchArrayFromDBWithEntity:(NSString*)entityName forKey:(NSString*)keyName withPredicate:(NSPredicate*)predicate inContext:(NSManagedObjectContext *)moc;
- (NSManagedObjectContext *) managedObjectContext;

// Class methods

+ (BOOL)saveContext:(NSManagedObjectContext *)managedObjectContext;
+ (id)sharedInstance;

+ (BOOL)checkIfUserExistsForLogin:(NSString *)userName password:(NSString *)password inManagedObjectContext:(NSManagedObjectContext *)managedObjectContext;

+ (void)saveUserToCoreData:(NSArray *)user inManagedObjectContext:(NSManagedObjectContext *)managedObjectContext;

+ (void)saveSchoolsToCoreData:(NSArray *)schools inManagedObjectContext:(NSManagedObjectContext *)managedObjectContext;

+ (void)saveCourseToCoreData:(NSDictionary *)course forUser:(User *)user inManagedObjectContext:(NSManagedObjectContext *)managedObjectContext;

+ (void)saveNewsToCoreData:(NSArray *)newsArray forCourse:(Course *)course inManagedObjectContext:(NSManagedObjectContext *)managedObjectContext;

+ (School *)getSchoolFromCoreData:(NSString *) shortName inManagedObjectContext:(NSManagedObjectContext *)managedObjectContext;
+ (User *)getUserFromCoreData:(NSString *) email inManagedObjectContext:(NSManagedObjectContext *)managedObjectContext;
+ (Course *)getCourseFromCoreData:(NSString *)courseId inManagedObjectContext:(NSManagedObjectContext *)managedObjectContext;

@end