//
//  CoreDataSingleton.m
//  EducationPlus
//
//  Created by Jon Freberg on 3/23/12.
//  Copyright (c) 2012 FrebergWeb. All rights reserved.
//
//  This class is the singleton class that makes the NSManagedObjectContext object are
//  always used in the correct thread.
//

#import "CoreDataSingleton.h"
#import "EducationPlusLib.h"
#import "UserWS.h"
#import "User+Create.h"
#import "School+Create.h"
#import "Course+Create.h"
#import "News+Create.h"

@implementation CoreDataSingleton

static CoreDataSingleton *sharedInstance = nil;

+ (CoreDataSingleton *)sharedInstance {
    NSLog(@"sharedInstance kalt");
    if (nil != sharedInstance) {
        return sharedInstance;
    }
    
    static dispatch_once_t pred;
    dispatch_once(&pred, ^{
        sharedInstance = [[CoreDataSingleton alloc] init];
    });
    
    return sharedInstance;
}

- (NSManagedObjectContext *) managedObjectContext {
    NSLog(@"managedObjectContext kalt");
    if (managedObjectContext != nil) {
        return managedObjectContext;
    }
    
    NSPersistentStoreCoordinator *coordinator = [self persistentStoreCoordinator];
    if (coordinator != nil) {
        managedObjectContext = [[NSManagedObjectContext alloc] init];
        [managedObjectContext setPersistentStoreCoordinator: coordinator];
    }
    return managedObjectContext;
}

- (NSManagedObjectModel *)managedObjectModel {
    NSLog(@"managedObjectModel kalt");
    if (managedObjectModel != nil) {
        return managedObjectModel;
    }
    managedObjectModel = [NSManagedObjectModel mergedModelFromBundles:nil];
    return managedObjectModel;
}

- (NSPersistentStoreCoordinator *)persistentStoreCoordinator {
    NSLog(@"persistenStoreCoordinator kalt");
    if (persistentStoreCoordinator != nil) {
        return persistentStoreCoordinator;
    }
    
    NSURL *storeUrl = [NSURL fileURLWithPath: [[self applicationDocumentsDirectory] stringByAppendingPathComponent: @"EducationPlus.sqlite"]];
    
    NSError *error;
    persistentStoreCoordinator = [[NSPersistentStoreCoordinator alloc] initWithManagedObjectModel: [self managedObjectModel]];
    if (![persistentStoreCoordinator addPersistentStoreWithType:NSSQLiteStoreType configuration:nil URL:storeUrl options:nil error:&error]) {
        // Handle the error.
    }
    
    return persistentStoreCoordinator;
}

- (NSMutableArray*)fetchArrayFromDBWithEntity:(NSString*)entityName forKey:(NSString*)keyName withPredicate:(NSPredicate*)predicate inContext:(NSManagedObjectContext *)moc
{
    NSFetchRequest *request = [[NSFetchRequest alloc] init];
    NSEntityDescription *entity = [NSEntityDescription entityForName:entityName inManagedObjectContext:moc];
    [request setEntity:entity];
    NSSortDescriptor *sortDescriptor = [[NSSortDescriptor alloc] initWithKey:keyName ascending:NO];
    NSArray *sortDescriptors = [[NSArray alloc] initWithObjects:sortDescriptor, nil];
    [request setSortDescriptors:sortDescriptors];
    
    if (predicate != nil)
        [request setPredicate:predicate];
    
    NSError *error = nil;
    NSMutableArray *mutableFetchResults = [[managedObjectContext executeFetchRequest:request error:&error] mutableCopy];
    if (mutableFetchResults == nil) {
        NSLog(@"%@", error);
    }
    return mutableFetchResults;
}

+ (BOOL)saveContext:(NSManagedObjectContext *)managedObjectContext {
    NSLog(@"saveContext kalt");
	if (![managedObjectContext hasChanges])
		return YES;
    
	NSError *error = nil;
	if (![managedObjectContext save:&error]) {
		NSLog(@"Error while saving: %@\n%@", [error localizedDescription], [error userInfo]);
		return NO;
	}
    
	return YES;
}

+ (void)saveUserToCoreData:(NSArray *)user inManagedObjectContext:(NSManagedObjectContext *)managedObjectContext {
    NSLog(@"saveUserToCoreData called");
    NSLog(@"Users email: %@", [user valueForKey:@"email"]);
    NSLog(@"USER FRA SAVE: %@", user);
    [User userWithEmail:user inManagedObjectContext:managedObjectContext];
    [self saveContext:managedObjectContext];
}

+ (void)saveCourseToCoreData:(NSDictionary *)course forUser:(User *)user inManagedObjectContext:(NSManagedObjectContext *)managedObjectContext {
    NSLog(@"saveCourseToCoreData called");
    [Course courseForUser:user courses:course inManagedObjectContext:managedObjectContext];
    [self saveContext:managedObjectContext];
}

+ (void)saveNewsToCoreData:(NSArray *)newsArray forCourse:(Course *)course inManagedObjectContext:(NSManagedObjectContext *)managedObjectContext {
    NSLog(@"saveNewsToCoreData called");
    for(NSDictionary *news in newsArray) {
        [News newsForCourse:news forCourse:course inManagedObjectContext:managedObjectContext];
    }
    [self saveContext:managedObjectContext];
}

+ (void)saveSchoolsToCoreData:(NSArray *)schools inManagedObjectContext:(NSManagedObjectContext *)managedObjectContext {
    NSLog(@"Save Schools To Core Data");
    for(NSDictionary *s in schools) {
        [School schoolsWithWSInfo:s inManagedObjectContext:managedObjectContext];
    }
    [self saveContext:managedObjectContext];
    
}

+ (BOOL)checkIfUserExistsForLogin:(NSString *)userName password:(NSString *)password inManagedObjectContext:(NSManagedObjectContext *)managedObjectContext {
    BOOL userExists = NO;
    NSFetchRequest *request = [NSFetchRequest fetchRequestWithEntityName:@"User"];
    request.predicate = [NSPredicate predicateWithFormat:@"email = %@", userName];
    NSSortDescriptor *sortDescriptor = [NSSortDescriptor sortDescriptorWithKey:@"email" ascending:YES];
    request.sortDescriptors = [NSArray arrayWithObject:sortDescriptor];

    [request setReturnsObjectsAsFaults:NO]; // To make the NSLog of the object human readable!
    
    NSError *error = nil;
    NSArray *userArray = [managedObjectContext executeFetchRequest:request error:&error];
    
    User *user = nil;
    
    if([userArray count] == 1) {
        user = [userArray lastObject];
        NSLog(@"USERNAME: %@", user.email);
        NSLog(@"PASSWORD: %@", user.password);
        
    }
        
    if(([userName isEqualToString:user.email]) && ([[EducationPlusLib sha1:password] isEqualToString:user.password])) {
        NSLog(@"USER EXISTS.");
        userExists = YES;
    }
    return userExists;
}

+ (School *)getSchoolFromCoreData:(NSString *) shortName inManagedObjectContext:(NSManagedObjectContext *)managedObjectContext {
    NSFetchRequest *request = [NSFetchRequest fetchRequestWithEntityName:@"School"];
    request.predicate = [NSPredicate predicateWithFormat:@"shortName = %@", shortName];
    NSSortDescriptor *sortDescriptor = [NSSortDescriptor sortDescriptorWithKey:@"shortName" ascending:YES];
    request.sortDescriptors = [NSArray arrayWithObject:sortDescriptor];
    
    [request setReturnsObjectsAsFaults:NO];
    
    NSError *error = nil;
    NSArray *schoolArray = [managedObjectContext executeFetchRequest:request error:&error];
    
    School *school = nil;
    
    if([schoolArray count] == 1) {
        school = [schoolArray lastObject];
    }
    return school;
}

+ (User *)getUserFromCoreData:(NSString *) email inManagedObjectContext:(NSManagedObjectContext *)managedObjectContext {
    NSFetchRequest *request = [NSFetchRequest fetchRequestWithEntityName:@"User"];
    request.predicate = [NSPredicate predicateWithFormat:@"email = %@", email];
    NSSortDescriptor *sortDescriptor = [NSSortDescriptor sortDescriptorWithKey:@"email" ascending:YES];
    request.sortDescriptors = [NSArray arrayWithObject:sortDescriptor];
    
    [request setReturnsObjectsAsFaults:NO];
    
    NSError *error = nil;
    NSArray *userArray = [managedObjectContext executeFetchRequest:request error:&error];
    
    User *user = nil;
    
    if([userArray count] == 1) {
        user = [userArray lastObject];
    }
    return user;
}

+ (Course *)getCourseFromCoreData:(NSString *)courseId inManagedObjectContext:(NSManagedObjectContext *)managedObjectContext {
    NSFetchRequest *request = [NSFetchRequest fetchRequestWithEntityName:@"Course"];
    request.predicate = [NSPredicate predicateWithFormat:@"unique = %@", courseId];
    NSSortDescriptor *sortDescriptor = [NSSortDescriptor sortDescriptorWithKey:@"unique" ascending:YES];
    request.sortDescriptors = [NSArray arrayWithObject:sortDescriptor];
    
    [request setReturnsObjectsAsFaults:NO];
    
    NSError *error = nil;
    NSArray *courseArray = [managedObjectContext executeFetchRequest:request error:&error];
    
    Course *course = nil;
    
    if([courseArray count] == 1) {
        course = [courseArray lastObject];
    }
    return course;
}

#pragma mark -
#pragma mark Application's documents directory

/**
 Returns the path to the application's documents directory.
 */
- (NSString *)applicationDocumentsDirectory {
    NSLog(@"applicationDocumentsDirectory kalt");
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *basePath = ([paths count] > 0) ? [paths objectAtIndex:0] : nil;
    return basePath;
}

@end














