//
//  Course+Create.m
//  EducationPlus
//
//  Created by Jon Freberg on 5/14/12.
//  Copyright (c) 2012 FrebergWeb. All rights reserved.
//
//  Category class created on the Course object. This method fetches a course object from the server
//  and saves it locally if it does not exist there.
//

#import "Course+Create.h"

@implementation Course (Create)

+ (Course *)courseForUser:(User *)user courses:(NSDictionary *)courseDictionary inManagedObjectContext:(NSManagedObjectContext *)context {
    Course *course = nil;
    
    NSFetchRequest *request = [NSFetchRequest fetchRequestWithEntityName:@"Course"];

    request.predicate = [NSPredicate predicateWithFormat:@"unique = %@", [courseDictionary valueForKey:@"courseId"]];
    
    NSSortDescriptor *sortDescriptor = [NSSortDescriptor sortDescriptorWithKey:@"unique" ascending:YES];
    request.sortDescriptors = [NSArray arrayWithObject:sortDescriptor];
    
    NSError *error = nil;
    NSArray *courseArray = [context executeFetchRequest:request error:&error];
    
    if(!courseArray || [courseArray count] > 1) {
        NSLog(@"ERROR trying to fetch course from Core Data. courseArray length: %@", [courseArray count]);
    } else if ([courseArray count] == 0) {
        course = (Course *)[NSEntityDescription insertNewObjectForEntityForName:@"Course" inManagedObjectContext:context];
        course.unique = [[NSNumber alloc] initWithInt:[[courseDictionary valueForKey:@"courseId"] integerValue]];
        course.code = [courseDictionary valueForKey:@"courseCode"];
        course.name = [courseDictionary valueForKey:@"courseName"];
        course.descriptionCourse = [courseDictionary valueForKey:@"courseDescription"];
        NSSet *userSet = [NSSet setWithObject:user];
        course.students = userSet;
    } else {
        course = [courseArray lastObject];
        NSSet *userSet = [NSSet setWithObject:user];
        course.students = userSet;
    }
    return course;
}

@end
