//
//  CourseTableViewController.m
//  EducationPlus
//
//  Created by Jon Freberg on 5/12/12.
//  Copyright (c) 2012 FrebergWeb. All rights reserved.
//
//  This class is what handles the fetching of course objects from Core Data
//  and adds them to the Course Table View.
//

#import "CourseTableViewController.h"
#import "Course+Create.h"

@interface CourseTableViewController ()

@end

@implementation CourseTableViewController

@synthesize user = _user;
@synthesize course = _course;

- (void)setupFetchedResultsController:(NSManagedObjectContext *)managedObjectContext // attaches an NSFetchRequest to this UITableViewController
{
    NSLog(@"setupFetchedResultsController ER KALT");
    NSFetchRequest *request = [NSFetchRequest fetchRequestWithEntityName:@"Course"];
    request.sortDescriptors = [NSArray arrayWithObject:[NSSortDescriptor sortDescriptorWithKey:@"unique" ascending:YES selector:@selector(localizedCaseInsensitiveCompare:)]];

    request.predicate = [NSPredicate predicateWithFormat:@"ANY students.unique = %@", self.user.unique];
    
    self.fetchedResultsController = [[NSFetchedResultsController alloc] initWithFetchRequest:request
                                                                        managedObjectContext:managedObjectContext
                                                                          sectionNameKeyPath:nil
                                                                                   cacheName:nil];
}

- (void)loadCoursesForUser:(User *)user inManagedObjectContext:(NSManagedObjectContext *)managedObjectContext {
    
    
    
    [self setupFetchedResultsController:managedObjectContext];
}

#pragma mark - UITableView Delegates


- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    [tableView deselectRowAtIndexPath:indexPath animated:YES];

    self.course = [self.fetchedResultsController objectAtIndexPath:indexPath];
    NSString *name = self.course.name;
    NSString *desc = self.course.descriptionCourse;
    
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:name message:desc delegate:self cancelButtonTitle:@"Ok" otherButtonTitles:nil];
    [alert show];
    
}


- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    NSLog(@"cellForRowAtIndexPath ER KALT FOR COURSE");
    static NSString *CellIdentifier = @"Course Cell";
    
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) {
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier];
    }
    
    // ask NSFetchedResultsController for the NSMO at the row in question
    Course *course = [self.fetchedResultsController objectAtIndexPath:indexPath];
    
    // Then configure the cell using it ...
    cell.textLabel.text = course.name;
    cell.detailTextLabel.text = course.code;
    
    cell.opaque = YES;

    return cell;  
}

@end