//
//  ViewController.m
//  EducationPlus
//
//  Created by Jon Freberg on 3/7/12.
//  Copyright (c) 2012 FrebergWeb. All rights reserved.
//
//  Some NSLogs are not removed for debugging purposes for future developers
//

#import <QuartzCore/QuartzCore.h>
#import "LoginViewController.h"
#import "EducationPlusLib.h"
#import "UserWS.h"
#import "KeychainItemWrapper.h"
#import "User+Create.h"
#import "Course+Create.h"

@interface LoginViewController()
@property (nonatomic, weak) IBOutlet UIToolbar *toolbar;
@end

@implementation LoginViewController

@synthesize splitViewBarButtonItem = _splitViewBarButtonItem;
@synthesize toolbar = _toolbar;
@synthesize UserNameTextField;
@synthesize PasswordTextField;
@synthesize ResponseTextLabel;
@synthesize LoginButton;
@synthesize managedObjectContext = _managedObjectContext;


- (IBAction)LogInButtonAction:(id)sender {
    /* The login button action method checks for the following possibilities when trying to login:
     1. Use KeyChainItems
     2. Check if user exists i Core Data
        2.1. If user exists in Core Data -> Save to KeyChain
     3. Check if user exists with call to web service tryLoginCred
        3.1. If user exists at server -> Save to Core Data -> Save to KeyChain
     4. If user didnt exist, present user with wrong username and/or password
     
     5. If login is OK -> Updates from server will be fetched
     */
    
    [self startActivitySpinner];
    NSString *username = UserNameTextField.text;
    NSString *password = PasswordTextField.text;
    KeychainItemWrapper *keychainItem = [[KeychainItemWrapper alloc] initWithIdentifier:@"EducationPlusKeyChain" accessGroup:nil];
    NSString *keychainUsername = [keychainItem objectForKey:(__bridge id)kSecValueData];
    NSString *keychainPassword = [keychainItem objectForKey:(__bridge id)kSecAttrAccount];
    
    /* 1 - Logg in with Keychain */
    if(([username isEqualToString:keychainUsername]) && (![keychainUsername isEqualToString:@""])
       && ([password isEqualToString:keychainPassword]) && (![keychainPassword isEqualToString:@""])) {
        dispatch_queue_t checkLoginQueue = dispatch_queue_create("check login", NULL);
        dispatch_async(checkLoginQueue, ^{
            NSLog(@"Logg inn: KEYCHAIN");
            dispatch_async(dispatch_get_main_queue(), ^{
                [self performSegueWithIdentifier:@"LoginSegue" sender:self];
            });
        });
        dispatch_release(checkLoginQueue);

    /* 2 - Logg in with Core Data */
    } else if ([CoreDataSingleton checkIfUserExistsForLogin:username password:password inManagedObjectContext:managedObjectContext]) {
        dispatch_queue_t checkLoginQueue = dispatch_queue_create("check login", NULL);
        dispatch_async(checkLoginQueue, ^{
            NSLog(@"Logg inn: CORE DATA");
            dispatch_async(dispatch_get_main_queue(), ^{
                [keychainItem resetKeychainItem];
                [keychainItem setObject:username forKey:(__bridge id)kSecValueData];
                [keychainItem setObject:password forKey:(__bridge id)kSecAttrAccount];
                
                [self stopActivitySpinner];
                [self performSegueWithIdentifier:@"LoginSegue" sender:self];

            });
        });
        dispatch_release(checkLoginQueue);
           /* 3 - Logg in with Web Service */
    } else if (!([CoreDataSingleton checkIfUserExistsForLogin:username password:password inManagedObjectContext:managedObjectContext])) {
        NSLog(@"Logg inn: WEB SERVICE");
        dispatch_queue_t checkLoginQueue = dispatch_queue_create("check login", NULL);
        dispatch_async(checkLoginQueue, ^{
            NSArray *userArray = [UserWS tryLoginCred:username password:[EducationPlusLib sha1:password]];
            dispatch_async(dispatch_get_main_queue(), ^{
                NSLog(@"USER ARRAY INFO ======================");
                NSLog(@"userArray: %@", userArray);
                NSLog(@"USER ARRAY INFO ======================");
                if(userArray) {
                    User *user = user = [User userWithEmail:userArray inManagedObjectContext:managedObjectContext];
                    NSArray *courseList = [userArray valueForKey:@"courseListJSON"];
                
                    if([courseList count] > 0) {
                        for(NSDictionary *courses in courseList) {
                            [CoreDataSingleton saveCourseToCoreData:courses forUser:user inManagedObjectContext:managedObjectContext];      // Denne må legge til users hvis course finnes.
                        }
                    }
                    [keychainItem resetKeychainItem];
                    [keychainItem setObject:username forKey:(__bridge id)kSecValueData];
                    [keychainItem setObject:password forKey:(__bridge id)kSecAttrAccount];
                    
                    [self stopActivitySpinner];
                    [self performSegueWithIdentifier:@"LoginSegue" sender:self];
                } else {
                    NSLog(@"userArray skal vare null: %@", userArray);
                    [self stopActivitySpinner];
                    ResponseTextLabel.text = @"Wrong username and/or password.";
                }
            });
        });
        
        dispatch_release(checkLoginQueue);

    } else {
        [self stopActivitySpinner];
        ResponseTextLabel.text = @"Wrong username and/or password.";
    }
}

- (void)startActivitySpinner {
    [self.view addSubview:bgView];
    [ai startAnimating];
    [UIApplication sharedApplication].networkActivityIndicatorVisible = YES;
}

- (void)stopActivitySpinner {
    [ai stopAnimating];
    [bgView removeFromSuperview];
    [UIApplication sharedApplication].networkActivityIndicatorVisible = NO;
}

- (void)targetMethod:(NSTimer *)theTimer {
    NSLog(@"VENTET FOR SPINNER");
}

// Prepares the user and sets the UserViewControllers user object to be the one that logged in
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    User *user = [CoreDataSingleton getUserFromCoreData:UserNameTextField.text inManagedObjectContext:managedObjectContext];
    NSLog(@"Prover aa sette User med selector");
    if ([segue.destinationViewController respondsToSelector:@selector(setUser:)]) {
        [segue.destinationViewController performSelector:@selector(setUser:) withObject:user];
        NSLog(@"Segue about to be performed");
        [self stopActivitySpinner];
    }
}

- (void)handleSplitViewBarButtonItem:(UIBarButtonItem *)splitViewBarButtonItem {
    NSMutableArray *toolbarItems = [self.toolbar.items mutableCopy];
    if (_splitViewBarButtonItem) [toolbarItems removeObject:_splitViewBarButtonItem];
    if (splitViewBarButtonItem) [toolbarItems insertObject:splitViewBarButtonItem atIndex:0];
    self.toolbar.items = toolbarItems;
    _splitViewBarButtonItem = splitViewBarButtonItem;
}

- (void)setSplitViewBarButtonItem:(UIBarButtonItem *)splitViewBarButtonItem {
    if (splitViewBarButtonItem != _splitViewBarButtonItem) {
        [self handleSplitViewBarButtonItem:splitViewBarButtonItem];
    }
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
}

#pragma mark - View lifecycle

- (void)viewWillAppear:(BOOL)animated {

    [super viewWillAppear:animated];
    KeychainItemWrapper *keychainItem = [[KeychainItemWrapper alloc] initWithIdentifier:@"EducationPlusKeyChain" accessGroup:nil];
    NSString *username = [keychainItem objectForKey:(__bridge id)kSecValueData];    // username
    NSString *password = [keychainItem objectForKey:(__bridge id)kSecAttrAccount];  // password
    
    if([username isEqualToString:@""]) {
        NSLog(@"Keychain EMPTY");
    } else {
        NSLog(@"Username: %@", username);
        NSLog(@"Password: %@", password);
        UserNameTextField.text = username;
        PasswordTextField.text = password;
    }
}



- (void)viewDidLoad {
    [super viewDidLoad];
    
    // Creating loging in activity window
    
    CGPoint center = CGPointMake(self.view.frame.size.width/2,self.view.frame.size.height/2);
    
    CGRect frame = CGRectMake(center.x-115, center.y-135, 230, 150);
    bgView = [[UIView alloc] initWithFrame:frame];
    bgView.backgroundColor = [UIColor blackColor];
    bgView.layer.cornerRadius = 10;
    bgView.alpha = 0.8;
    
    CGRect labelFrame = CGRectMake(0, 80, 230, 50);
    
    UILabel *loggingIn = [[UILabel alloc] initWithFrame:labelFrame];
    loggingIn.text = @"Logging in...";
    loggingIn.font = [UIFont fontWithName:@"Helvetica" size:20];
    loggingIn.textAlignment = UITextAlignmentCenter;
    loggingIn.textColor = [UIColor whiteColor];
    loggingIn.backgroundColor = [UIColor clearColor];
    
    CGRect aiFrame = CGRectMake(90, 0, 50, 100);
    ai = [[UIActivityIndicatorView alloc]initWithFrame:aiFrame];
    ai.activityIndicatorViewStyle = UIActivityIndicatorViewStyleWhiteLarge;
    
    [bgView addSubview:ai];
    [bgView addSubview:loggingIn];
    
    PasswordTextField.secureTextEntry = YES;
    [self handleSplitViewBarButtonItem:self.splitViewBarButtonItem];
    managedObjectContext = [[CoreDataSingleton sharedInstance] managedObjectContext];
}

- (void)viewDidUnload {
    [self setLoginButton:nil];
    [super viewDidUnload];
}

- (void)willRotateToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation duration:(NSTimeInterval)duration {
    NSLog(@"DEVICE HAR ROTERT");
    if (UIInterfaceOrientationIsLandscape(toInterfaceOrientation)) {
        NSLog(@"DEVICE ER I LANDSCAPE");
    }
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

@end
