//
//  News+Create.m
//  EducationPlus
//
//  Created by Jon Freberg on 5/14/12.
//  Copyright (c) 2012 FrebergWeb. All rights reserved.
//
//  Category class created on the News object. This method fetches a news object from the server
//  and saves it locally if it does not exist there.
//

#import "News+Create.h"

@implementation News (Create)

+ (News *)newsForCourse:(NSDictionary *)newsDictionary forCourse:(Course *)course inManagedObjectContext:(NSManagedObjectContext *)context {
    News *news = nil;
    NSFetchRequest *request = [NSFetchRequest fetchRequestWithEntityName:@"News"];
    request.predicate = [NSPredicate predicateWithFormat:@"unique = %@", [newsDictionary valueForKey:@"newsId"]];
    
    NSSortDescriptor *sortDescriptor = [NSSortDescriptor sortDescriptorWithKey:@"unique" ascending:NO];
    request.sortDescriptors = [NSArray arrayWithObject:sortDescriptor];
    
    NSError *error = nil;
    NSArray *newsArray = [context executeFetchRequest:request error:&error];
    
    if (!newsArray || [newsArray count] > 1) {
        NSLog(@"ERROR trying to fetch news from Core Data. newsArray length: %@", [newsArray count]);
    } else if ([newsArray count] == 0) {
        news = (News *)[NSEntityDescription insertNewObjectForEntityForName:@"News" inManagedObjectContext:context];
        news.unique = [[NSNumber alloc] initWithInt:[[newsDictionary valueForKey:@"newsId"] integerValue]];
        news.topic = [newsDictionary valueForKey:@"topic"];
        news.text = [newsDictionary valueForKey:@"text"];
        double dateInDouble = [[newsDictionary valueForKey:@"regTime"] doubleValue];
        NSDate *date = [NSDate dateWithTimeIntervalSinceReferenceDate:dateInDouble];
        news.regTime = date;
        news.belongsTo = course;
        
        NSLog(@"News saved to Core Data: %@", news);
    } else {
        news = [newsArray lastObject];
    }
    return  news;
}

@end
