//
//  RegisterViewController.m
//  Portal
//
//  Created by Jon Freberg on 2/23/12.
//  Copyright (c) 2012 Jon Freberg. All rights reserved.
//

#import "UserWS.h"
#import "School+Create.h"
#import "CoreDataSingleton.h"
#import "EducationPlusLib.h"
#import "RegisterViewController.h"
#import "KeychainItemWrapper.h"

@interface RegisterViewController()
@property (nonatomic, weak) IBOutlet UIToolbar *toolbar;
@end

@implementation RegisterViewController
@synthesize FirstNameTextField;
@synthesize LastNameTextField;
@synthesize GradYearTextField;
@synthesize EmailTextField;
@synthesize PasswordTextField;
@synthesize PasswordResponseLabel;
@synthesize FirstNameResponseLabel;
@synthesize LastNameResponseLabel;
@synthesize GraduationYearResponseLabel;
@synthesize EmailResponseLabel;
@synthesize ResponseLabel;
@synthesize toolbar = _toolbar;
@synthesize splitViewBarButtonItem = _splitViewBarButtonItem;
@synthesize selectedSchool;
@synthesize searchResults;

@synthesize managedObjectContext = _managedObjectContext;

- (void)viewDidLoad {
    PasswordTextField.secureTextEntry = YES;
    managedObjectContext = [[CoreDataSingleton sharedInstance] managedObjectContext];
    self.searchResults = [NSMutableArray arrayWithCapacity:[[self.fetchedResultsController fetchedObjects] count]]; 
    CGRect searchFrame = CGRectMake(198, 564, 373.0, 44.0);
    self.searchDisplayController.searchBar.frame = searchFrame;
    
}

- (void)setupFetchedResultsController
{
    // 1 - Decide what Entity you want
    NSString *entityName = @"School"; // Put your entity name here
    NSLog(@"Setting up a Fetched Results Controller for the Entity named %@", entityName);
    
    // 2 - Request that Entity
    NSFetchRequest *request = [NSFetchRequest fetchRequestWithEntityName:entityName];
    
    // 3 - Filter it if you want
    //request.predicate = [NSPredicate predicateWithFormat:@"Role.name = Blah"];
    
    // 4 - Sort it if you want
    request.sortDescriptors = [NSArray arrayWithObject:[NSSortDescriptor sortDescriptorWithKey:@"unique" ascending:YES selector:@selector(localizedCaseInsensitiveCompare:)]];
    // 5 - Fetch it
    self.fetchedResultsController = [[NSFetchedResultsController alloc] initWithFetchRequest:request
                                                                        managedObjectContext:managedObjectContext
                                                                          sectionNameKeyPath:nil
                                                                                   cacheName:nil];
}

- (void)updateSchools {
    NSArray *schools = [UserWS getAllSchools];
    
    for (NSDictionary *schoolArrayObject in schools) {
        [School schoolsWithWSInfo:schoolArrayObject inManagedObjectContext:managedObjectContext];
    }
    [CoreDataSingleton saveContext:managedObjectContext];
    [self setupFetchedResultsController];
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    [FirstNameTextField becomeFirstResponder];
    [self updateSchools];
}

- (void)viewDidUnload {
    [self setFirstNameTextField:nil];
    [self setLastNameTextField:nil];
    [self setGradYearTextField:nil];
    [self setEmailTextField:nil];
    [self setPasswordTextField:nil];
    [self setResponseLabel:nil];
    [self setFirstNameResponseLabel:nil];
    [self setLastNameResponseLabel:nil];
    [self setGraduationYearResponseLabel:nil];
    [self setEmailResponseLabel:nil];
    [self setPasswordResponseLabel:nil];
    self.searchResults = nil;
    [self setSearchResults:nil];
    [super viewDidUnload];
}

- (IBAction)RegisterButtonAction:(id)sender {
    School *school = [CoreDataSingleton getSchoolFromCoreData:self.searchDisplayController.searchBar.text inManagedObjectContext:managedObjectContext];
    NSLog(@"School object fra core data: %@", school);
    
    if(school) {
        NSDictionary *schoolAsJsonDictionary = [EducationPlusLib generateSchoolJSONObject:[school.unique stringValue] shortName:school.shortName fullName:school.fullName];
        NSArray *user = [UserWS registerNewUser:[EducationPlusLib generateJSONObject:FirstNameTextField.text
                                                                            lastName:LastNameTextField.text
                                                                            gradYear:GradYearTextField.text
                                                                              school:schoolAsJsonDictionary
                                                                               email:EmailTextField.text
                                                                            password:PasswordTextField.text
                                                                     registerNewUser:YES]];
        if(!user) {
            NSLog(@"User finnes fra før.");
            ResponseLabel.text = @"User with that email already exists.";
        } else {
            ResponseLabel.text = @"User created. You will be forwarded to your home screen.";
            
            KeychainItemWrapper *keychainItem = [[KeychainItemWrapper alloc] initWithIdentifier:@"EducationPlusKeyChain" accessGroup:nil];
            [keychainItem resetKeychainItem];
            [keychainItem setObject:EmailTextField.text forKey:(__bridge id)kSecValueData];
            [keychainItem setObject:PasswordTextField.text forKey:(__bridge id)kSecAttrAccount];
            [CoreDataSingleton saveUserToCoreData:user inManagedObjectContext:managedObjectContext];
            
            [NSTimer scheduledTimerWithTimeInterval:2.0
                                             target:self
                                           selector:@selector(targetMethod:)
                                           userInfo:nil
                                            repeats:NO];
        }
    } else {
        NSLog(@"School objekt er NULL");
        ResponseLabel.text = @"The selected school doesnt exist.";
    }
}

- (BOOL)sendRegistrationEmail:(NSString *)to body:(NSString *)body {
    return YES;
}

- (void)targetMethod:(NSTimer *)theTimer {
    [self performSegueWithIdentifier:@"LoginSuccessfullSegue" sender:self];
}

- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    User *user = [CoreDataSingleton getUserFromCoreData:EmailTextField.text inManagedObjectContext:managedObjectContext];
    NSLog(@"Prover aa sette User med selector");
    if ([segue.destinationViewController respondsToSelector:@selector(setUser:)]) {
        [segue.destinationViewController performSelector:@selector(setUser:) withObject:user];
        NSLog(@"Segue about to be performed");
    }
}


/* DELEGATE METHODS FOR SEARCH BAR */

// These methods fetch school objects from Core Data and presents them in the table view given the string the user enters.

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    School *s = [self.searchResults objectAtIndex:indexPath.row];
    NSLog(@"School selected: %@", s);
    self.searchDisplayController.searchBar.text = s.shortName;
    tableView.hidden = YES;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString *CellIdentifier = @"School Cell";
    
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];

    if (cell == nil) {
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier];
    }
    
    // Configure the cell...
    School *school = nil;
	if (tableView == self.searchDisplayController.searchResultsTableView) {
        NSLog(@"Configuring cell to show search results");
        school = [self.searchResults objectAtIndex:indexPath.row];
    } else {
        NSLog(@"Configuring cell to show normal data");
        school = [self.fetchedResultsController objectAtIndexPath:indexPath];
    }
    cell.textLabel.text = school.shortName;
    return cell;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
	if (tableView == self.searchDisplayController.searchResultsTableView)
	{
        NSLog(@"Nr of rows: %d", [self.searchResults count]);
        return [self.searchResults count];
    }
	else
	{
        NSLog(@"Nr of rows: %d", [[[self.fetchedResultsController sections] objectAtIndex:section] numberOfObjects]);
        return [[[self.fetchedResultsController sections] objectAtIndex:section] numberOfObjects];
    }
}


#pragma mark -
#pragma mark Content Filtering

- (void)filterContentForSearchText:(NSString*)searchText scope:(NSString*)scope
{
	NSLog(@"Previous Search Results were removed.");
	[self.searchResults removeAllObjects]; 
    
	for (School *school in [self.fetchedResultsController fetchedObjects]) {
		if ([scope isEqualToString:@"All"] || [school.shortName isEqualToString:scope]) {
			NSComparisonResult result = [school.shortName compare:searchText 
                                                          options:(NSCaseInsensitiveSearch|NSDiacriticInsensitiveSearch) 
                                                            range:NSMakeRange(0, [searchText length])];
            if (result == NSOrderedSame) {
                NSLog(@"Adding school.shortName '%@' to searchResults as it begins with search text '%@'", school.shortName, searchText);
				[self.searchResults addObject:school];
            }
		}
	}
}

#pragma - Mine metoder


- (void)searchDisplayControllerWillBeginSearch:(UISearchDisplayController *)controller {
    NSLog(@"searchDisplayControllerWillBeginSearch");
}

- (void)searchBarTextDidBeginEditing:(UISearchBar *)searchBar {

}

- (void)searchDisplayControllerDidEndSearch:(UISearchDisplayController *)controller {
    CGRect searchFrame = CGRectMake(198, 564, 373.0, 44.0);
    controller.searchBar.frame = searchFrame;
}

- (void)searchDisplayControllerDidBeginSearch:(UISearchDisplayController *)controller {
    NSLog(@"searchDisplayControllerDidBeginSearch");
    CGRect tableFrame = CGRectMake(198, 608, 373, 194);
    controller.searchResultsTableView.frame = tableFrame;
    
    CGRect searchFrame = CGRectMake(198, 564, 373.0, 44.0);
    controller.searchBar.frame = searchFrame;
}

- (BOOL)searchBarShouldBeginEditing:(UISearchBar *)searchBar {
    NSLog(@"searchBarShouldBeginEditing kallt");
    CGRect searchFrame = CGRectMake(198, 564, 373.0, 44.0);
    searchBar.frame = searchFrame;

    return YES;
}


#pragma mark -
#pragma mark UISearchDisplayController Delegate Methods


- (BOOL)searchDisplayController:(UISearchDisplayController *)controller shouldReloadTableForSearchString:(NSString *)searchString
{
    NSLog(@"shouldReloadTableForSearchString called");
    [self filterContentForSearchText:searchString scope:@"All"];
    return YES;
}

- (BOOL)searchDisplayController:(UISearchDisplayController *)controller shouldReloadTableForSearchScope:(NSInteger)searchOption
{
    NSLog(@"shouldReloadTableForSearchScope called");
    [self filterContentForSearchText:[self.searchDisplayController.searchBar text] scope:@"All"];
    return YES;
}


// TODO: Methods for implementing validation of textfields are partially implemented below. For future development of application

- (void)textFieldDidEndEditing:(UITextField *)textField {
//    NSLog(@"textFieldDidEndEditing metode kjørt.");
//    
//    if(textField == FirstNameTextField) {
//        // First name invalid        
//        NSLog(@"FirstName: %@", textField.text);
//        if([self validateFirstNameWithString:textField.text]) {
//           FirstNameResponseLabel.text = @"OK"; 
//        } else if(![self validateFirstNameWithString:textField.text] && textField.text.length > 0) {
//            FirstNameResponseLabel.text = @"Only letters, A-Z";
//        }
//    } else if (textField == LastNameTextField) {
//        // Last name invalid
//        NSLog(@"LastName: %@", textField.text);        
//        if([self validateLastNameWithString:textField.text]) {
//            LastNameResponseLabel.text = @"OK";
//        } else if(![self validateLastNameWithString:textField.text] && textField.text.length > 0) {
//            LastNameResponseLabel.text = @"Only letters, A-Z";
//        }
//    } else if (textField == GradYearTextField) {
//        // Graduation year invalid
//        // TODO: KUN 4 SIFFER, FRA DETTE ÅRET OG FREMOVER
//        NSLog(@"Graduation Year: %@", textField.text);        
//        if([self validateGraduationYearWithInt:textField.text]) {
//            GraduationYearResponseLabel.text = @"OK";
//        } else if(![self validateGraduationYearWithInt:textField.text] && textField.text.length > 0) {
//            GraduationYearResponseLabel.text = @"Only numbers, 0-9";
//        }
//    } else if (textField == EmailTextField) {
//        // Email invalid
//        NSLog(@"Email: %@", textField.text);
//        if([self validateEmailWithString:textField.text]) {
//            EmailResponseLabel.text = @"OK";
//        } else if(![self validateEmailWithString:textField.text] && textField.text.length > 0) {
//            EmailResponseLabel.text = @"Invalid email";
//        }
//    } else if (textField == PasswordTextField) {
//        // Password invalid        
//        NSLog(@"Password: %@", textField.text);
//        if([self validatePasswordWithString:textField.text]) {
//            PasswordResponseLabel.text = @"OK";
//        } else if(![self validatePasswordWithString:textField.text] && textField.text.length > 0) {
//            PasswordResponseLabel.text = @"Only letters and numbers";
//        }
//    } else {
//        NSLog(@"Value: %@", textField.text);
//    }
}

- (void)handleSplitViewBarButtonItem:(UIBarButtonItem *)splitViewBarButtonItem {
    NSMutableArray *toolbarItems = [self.toolbar.items mutableCopy];
    if (_splitViewBarButtonItem) [toolbarItems removeObject:_splitViewBarButtonItem];
    if (splitViewBarButtonItem) [toolbarItems insertObject:splitViewBarButtonItem atIndex:1];
    self.toolbar.items = toolbarItems;
    _splitViewBarButtonItem = splitViewBarButtonItem;
}

- (void)setSplitViewBarButtonItem:(UIBarButtonItem *)splitViewBarButtonItem {
    if (splitViewBarButtonItem != _splitViewBarButtonItem) {
        [self handleSplitViewBarButtonItem:splitViewBarButtonItem];
    }
}

- (BOOL)validateFirstNameWithString:(NSString *)firstName {
    NSString *lettersOnlyRegex = @"^[a-zA-Z]{2,30}";
    NSPredicate *firstNameTest = [NSPredicate predicateWithFormat:@"SELF MATCHES %@", lettersOnlyRegex];
    return [firstNameTest evaluateWithObject:FirstNameTextField.text];
}

- (BOOL)validateLastNameWithString:(NSString *)lastName {
    NSString *lettersOnlyRegex = @"^[a-zA-Z]{2,30}";
    NSPredicate *lastNameTest = [NSPredicate predicateWithFormat:@"SELF MATCHES %@", lettersOnlyRegex];
    return [lastNameTest evaluateWithObject:FirstNameTextField.text];
}

- (BOOL)validateGraduationYearWithInt:(NSString *)graduationYear {
    NSString *numbersOnlyRegex = @"^[0-9]{4}";
    NSPredicate *graduationYearTest = [NSPredicate predicateWithFormat:@"SELF MATCHES %@", numbersOnlyRegex];
    return [graduationYearTest evaluateWithObject:graduationYear];
}

- (BOOL)validateEmailWithString:(NSString*)email {
    NSString *emailRegex = @"[A-Z0-9a-z._%+-]+@[A-Za-z0-9.-]+\\.[A-Za-z]{2,4}";
    NSPredicate *emailTest = [NSPredicate predicateWithFormat:@"SELF MATCHES %@", emailRegex]; 
    return [emailTest evaluateWithObject:email];
}

- (BOOL)validatePasswordWithString:(NSString *)password {
    NSString *numbersAndLettersOnlyRegex = @"^[a-zA-Z0-9]{4,12}";
    NSPredicate *passwordTest = [NSPredicate predicateWithFormat:@"SELF MATCHES %@", numbersAndLettersOnlyRegex];
    return [passwordTest evaluateWithObject:password];
}


@end



