//
//  UserViewController.m
//  EducationPlus
//
//  Created by Jon Freberg on 3/19/12.
//  Copyright (c) 2012 FrebergWeb. All rights reserved.
//

#import <QuartzCore/QuartzCore.h>
#import "UserViewController.h"
#import "KeychainItemWrapper.h"
#import "User.h"
#import "User+Create.h"
#import "UserWS.h"
#import "AppDelegate.h"
#import "EducationPlusLib.h"
#import "School+Create.h"

@interface UserViewController()

@property (nonatomic, weak) IBOutlet UIToolbar *toolbar;
- (void)initializeUserProfile:(NSString *)email inManagedObjectContext:(NSManagedObjectContext *)managedObjectContext;
- (void)initializeTableViews;
@end

@implementation UserViewController

@synthesize UserNameLabel = _UserNameLabel;
@synthesize SchoolLabel = _SchoolLabel;
@synthesize CoursesTableView = _CoursesTableView;
@synthesize NewsTableView = _NewsTableView;
@synthesize MessageTableView = _MessageTableView;
@synthesize LogoutButton = _LogoutButton;
@synthesize refreshButton = _refreshButton;
@synthesize splitViewBarButtonItem = _splitViewBarButtonItem;
@synthesize toolbar = _toolbar;
@synthesize managedObjectContext = _managedObjectContext;
@synthesize keychainItem = _keychainItem;
@synthesize user = _user;
@synthesize course = _course;
@synthesize news = _news;

// Initializes labels to display user information
- (void)initializeUserProfile:(NSString *)email inManagedObjectContext:(NSManagedObjectContext *)moc {
    NSString *fn = self.user.firstName;
    NSString *ln = self.user.lastName;
    NSString *fullName = [fn stringByAppendingFormat:@" %@", ln];
    self.UserNameLabel.text = fullName;
    School *s = self.user.goesTo;
    self.SchoolLabel.text = s.shortName;
}

- (void)setUser:(User *)user {
    _user = user;
}

- (void)setCourse:(Course *)course {
    _course = course;
}

// Sets this view to be the course- and newsTableView delegate and datasource
- (void)viewDidLoad {
    _keychainItem = [[KeychainItemWrapper alloc] initWithIdentifier:@"EducationPlusKeyChain" accessGroup:nil];
    managedObjectContext = [[CoreDataSingleton sharedInstance] managedObjectContext];
    [self handleSplitViewBarButtonItem:self.splitViewBarButtonItem];
    NSLog(@"self.user.EMAIL: %@", self.user.email);
    [self initializeUserProfile:self.user.email inManagedObjectContext:managedObjectContext];
    
    if(courseTableViewController == nil) {
        courseTableViewController = [[CourseTableViewController alloc] init];
        [self.CoursesTableView.layer setCornerRadius:10.0];
    }
    
    if(newsTableViewController == nil) {
        newsTableViewController = [[NewsTableViewController alloc] init];
        [self.NewsTableView.layer setCornerRadius:10.0];
    }
    
    self.CoursesTableView.dataSource = courseTableViewController;
    self.CoursesTableView.delegate = courseTableViewController;

    courseTableViewController.user = self.user;

    self.NewsTableView.dataSource = newsTableViewController;
    self.NewsTableView.delegate = newsTableViewController;
    
    [self updateCoursesForUser:self.user.unique];
    [self initializeTableViews];
    [self createActivityIndicator];
}

- (void)viewDidUnload {
    [self setToolbar:nil];
    [self setLogoutButton:nil];
    [self setUserNameLabel:nil];
    [self setSchoolLabel:nil];
    [self setCoursesTableView:nil];
    [self setNewsTableView:nil];
    [self setMessageTableView:nil];
    [self setRefreshButton:nil];
    [super viewDidUnload];
}


// Called when the refresh button is pressed. Updates both course- and news tables.
- (void)updateCoursesForUser:(NSNumber *)usrID {
    NSString *userId = [usrID stringValue];
    NSLog(@"USER ID == %@", userId);
    NSArray *userArray = [UserWS getUserById:userId];
    NSArray *courseList = [userArray valueForKey:@"courseListJSON"];
    NSLog(@"Course List USER VIEW: ");
    NSLog(@"=======================");
    NSLog(@"%@", courseList);
    
    if(!courseList) {
        NSLog(@"COURSE LIST ER NULL");
        newsTableViewController.course = nil;
    } else if([courseList count] > 0) {
        for(NSDictionary *courses in courseList) {
            NSLog(@"Course i FOR: %@", courses);
            [CoreDataSingleton saveCourseToCoreData:courses forUser:self.user inManagedObjectContext:managedObjectContext];
            NSArray *newsForCourse = [UserWS getNewsForCourse:[courses valueForKey:@"courseId"]];
            NSLog(@"NEWS WEB SERVICE RETUR newsForCourse: %@", newsForCourse);
            Course *c = [CoreDataSingleton getCourseFromCoreData:[courses valueForKey:@"courseId"] inManagedObjectContext:managedObjectContext];
            NSLog(@"Course debug: %@", c);
            newsTableViewController.course = c;
            [CoreDataSingleton saveNewsToCoreData:newsForCourse forCourse:c inManagedObjectContext:managedObjectContext];
            [newsTableViewController setupFetchedResultsController:managedObjectContext];
        }
    }
}


- (void)initializeTableViews {
    [courseTableViewController setupFetchedResultsController:managedObjectContext];
}

- (void)handleSplitViewBarButtonItem:(UIBarButtonItem *)splitViewBarButtonItem
{
    NSLog(@"Handle Split View kalt");
    NSMutableArray *toolbarItems = [self.toolbar.items mutableCopy];
    if (_splitViewBarButtonItem) [toolbarItems removeObject:_splitViewBarButtonItem];
    if (splitViewBarButtonItem) [toolbarItems insertObject:splitViewBarButtonItem atIndex:0];
    self.toolbar.items = toolbarItems;
    _splitViewBarButtonItem = splitViewBarButtonItem;
}

- (void)setSplitViewBarButtonItem:(UIBarButtonItem *)splitViewBarButtonItem
{
    if (splitViewBarButtonItem != _splitViewBarButtonItem) {
        [self handleSplitViewBarButtonItem:splitViewBarButtonItem];
    }
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

#pragma - BUTTON METHODS

- (IBAction)LogoutButtonPressed:(id)sender {
    
    [UserWS getUser:[_keychainItem objectForKey:(__bridge id)kSecValueData]];    
    [_keychainItem resetKeychainItem];
    [self performSegueWithIdentifier:@"LogoutSegue" sender:self];
    NSString *username = [_keychainItem objectForKey:(__bridge id)kSecValueData];    // username
    NSString *password = [_keychainItem objectForKey:(__bridge id)kSecAttrAccount];  // password
    NSLog(@"LOGOUT");
    NSLog(@"Username: %@", username);
    NSLog(@"Password: %@", password);
}

- (IBAction)RefreshButtonPressed:(id)sender {
    [self startActivitySpinner];
    dispatch_queue_t updateUserQueue = dispatch_queue_create("update course", NULL);
    dispatch_async(updateUserQueue, ^{
        
        courseTableViewController = nil;
        newsTableViewController = nil;
        if(courseTableViewController == nil) {
            courseTableViewController = [[CourseTableViewController alloc] init];
        }
        
        if(newsTableViewController == nil) {
            newsTableViewController = [[NewsTableViewController alloc] init];
        }
        
        self.CoursesTableView.dataSource = courseTableViewController;
        self.CoursesTableView.delegate = courseTableViewController;
        
        courseTableViewController.user = self.user;
        
        self.NewsTableView.dataSource = newsTableViewController;
        self.NewsTableView.delegate = newsTableViewController;
        
        [self updateCoursesForUser:self.user.unique];
        [self initializeTableViews];
        
        dispatch_async(dispatch_get_main_queue(), ^{    
            [self stopActivitySpinner];
        });
    });
    dispatch_release(updateUserQueue);
}


// Creates and start the Downloading data... activity indicator
- (void)startActivitySpinner {
    [self.view addSubview:bgView];
    [ai startAnimating];
    [UIApplication sharedApplication].networkActivityIndicatorVisible = YES;
}

// Stops the Downloading data... activity indicator
- (void)stopActivitySpinner {
    [ai stopAnimating];
    [bgView removeFromSuperview];
    [UIApplication sharedApplication].networkActivityIndicatorVisible = NO;
}

// Creates the activity indicator
- (void)createActivityIndicator {
    CGPoint center = CGPointMake(self.view.frame.size.width/2,self.view.frame.size.height/2);
    
    CGRect frame = CGRectMake(center.x-115, center.y-135, 230, 150);
    bgView = [[UIView alloc] initWithFrame:frame];
    bgView.backgroundColor = [UIColor blackColor];
    bgView.layer.cornerRadius = 10;
    bgView.alpha = 0.8;
    
    CGRect labelFrame = CGRectMake(0, 80, 230, 50);
    
    UILabel *loggingIn = [[UILabel alloc] initWithFrame:labelFrame];    //WithFrame:labelFrame
    loggingIn.text = @"Downloading new data...";
    loggingIn.font = [UIFont fontWithName:@"Helvetica" size:20];
    loggingIn.textAlignment = UITextAlignmentCenter;
    loggingIn.textColor = [UIColor whiteColor];
    loggingIn.backgroundColor = [UIColor clearColor];
    
    CGRect aiFrame = CGRectMake(90, 0, 50, 100);
    ai = [[UIActivityIndicatorView alloc]initWithFrame:aiFrame];
    ai.activityIndicatorViewStyle = UIActivityIndicatorViewStyleWhiteLarge;
    
    [bgView addSubview:ai];
    [bgView addSubview:loggingIn];
}


@end
