//
//  UserWS.m
//  EducationPlus
//
//  Created by Jon Freberg on 3/21/12.
//  Copyright (c) 2012 FrebergWeb. All rights reserved.
//
//  This class contains all the web service calls done from the application. All requests are implemented as 
//  NSURL synchronous requests. Some GET requests have information given in the URL, while all POST request
//  sets the HTTP body to be a JSON structured object.
//
//  Returns: Every method returns an NSArray with the object value returned from the server.
//
//

#import "UserWS.h"
#import "User.h"

@implementation UserWS

+ (NSArray *)tryLogin:(NSString *)jsonObject {
    NSMutableURLRequest *request = [NSMutableURLRequest requestWithURL:[NSURL URLWithString:@"http://vm-6115.idi.ntnu.no/abc/trylogin"]];
    [request addValue:@"application/json; charset=utf-8" forHTTPHeaderField:@"Content-Type"];
    [request setHTTPMethod:@"GET"];
    [request setHTTPBody:[jsonObject dataUsingEncoding:NSUTF8StringEncoding]];
    
    NSError *error = nil;
    NSURLResponse *response = nil;
    NSData *myConnection = [NSURLConnection sendSynchronousRequest:request returningResponse:&response error:&error];
    
    NSArray *results = myConnection ? [NSJSONSerialization JSONObjectWithData:myConnection options:0 error:&error] : nil;
    
    if (error) NSLog(@"[%@ %@] JSON error: %@", NSStringFromClass([self class]), NSStringFromSelector(_cmd), error.localizedDescription);
    NSLog(@"[%@ %@] received %@", NSStringFromClass([self class]), NSStringFromSelector(_cmd), results);
    return results;
}

+ (NSArray *)tryLoginCred:(NSString *)email password:(NSString *)password {
    NSString *urlRequest = [[NSString alloc] initWithFormat:@"http://vm-6115.idi.ntnu.no/abc/trylogin/%@/%@", email, password];
    NSMutableURLRequest *request = [NSMutableURLRequest requestWithURL:[NSURL URLWithString:urlRequest]];
    [request addValue:@"application/json; charset=utf-8" forHTTPHeaderField:@"Content-Type"];
    [request setHTTPMethod:@"GET"];
    
    NSError *error = nil;
    NSURLResponse *response = nil;
    NSData *myConnection = [NSURLConnection sendSynchronousRequest:request returningResponse:&response error:&error];
    
    NSArray *results = myConnection ? [NSJSONSerialization JSONObjectWithData:myConnection options:0 error:&error] : nil;
    
    if (error) NSLog(@"[%@ %@] JSON error: %@", NSStringFromClass([self class]), NSStringFromSelector(_cmd), error.localizedDescription);
    NSLog(@"[%@ %@] received %@", NSStringFromClass([self class]), NSStringFromSelector(_cmd), results);
    return results;
}

+ (NSArray *)registerNewUser:(NSString *)jsonObject {
    NSMutableURLRequest *request = [NSMutableURLRequest requestWithURL:[NSURL URLWithString:@"http://vm-6115.idi.ntnu.no/abc/fromipad/saveuser"]];
    NSString *postLength =  [NSString stringWithFormat:@"%d", [jsonObject length]];
    [request addValue:@"application/json; charset=utf-8" forHTTPHeaderField:@"Content-Type"];
    [request addValue:postLength forHTTPHeaderField:@"Content-Length"];    
    [request setHTTPMethod:@"POST"];
    [request setHTTPBody:[jsonObject dataUsingEncoding:NSUTF8StringEncoding]];
    
    NSError *error = nil;
    NSURLResponse *response = nil;
    NSData *myConnection = [NSURLConnection sendSynchronousRequest:request returningResponse:&response error:&error];
    
    NSArray *results = myConnection ? [NSJSONSerialization JSONObjectWithData:myConnection options:0 error:&error] : nil;
    
    if (error) NSLog(@"[%@ %@] JSON error: %@", NSStringFromClass([self class]), NSStringFromSelector(_cmd), error.localizedDescription);
    NSLog(@"[%@ %@] received %@", NSStringFromClass([self class]), NSStringFromSelector(_cmd), results);
    return results;
}

+ (NSArray *)getUser:(NSString *)email {
    NSString *urlRequest = [[NSString alloc] initWithFormat:@"http://vm-6115.idi.ntnu.no/abc/user/%@", email];
    NSMutableURLRequest *request = [NSMutableURLRequest requestWithURL:[NSURL URLWithString:urlRequest]];
    [request addValue:@"application/json; charset=utf-8" forHTTPHeaderField:@"Content-Type"];
    [request setHTTPMethod:@"GET"];

    NSError *error = nil;
    NSURLResponse *response = nil;
    NSData *myConnection = [NSURLConnection sendSynchronousRequest:request returningResponse:&response error:&error];
    NSArray *results = myConnection ? [NSJSONSerialization JSONObjectWithData:myConnection options:0 error:&error] : nil;

    if (error) NSLog(@"[%@ %@] JSON error: %@", NSStringFromClass([self class]), NSStringFromSelector(_cmd), error.localizedDescription);
    NSLog(@"[%@ %@] received %@", NSStringFromClass([self class]), NSStringFromSelector(_cmd), results);
    return results;
}

+ (NSArray *)getUserById:(NSString *)userId {
    NSString *urlRequest = [[NSString alloc] initWithFormat:@"http://vm-6115.idi.ntnu.no/abc/user/%@", userId];
    NSLog(@"URL REQUEST: %@", urlRequest);
    NSMutableURLRequest *request = [NSMutableURLRequest requestWithURL:[NSURL URLWithString:urlRequest]];
    [request addValue:@"application/json; charset=utf-8" forHTTPHeaderField:@"Content-Type"];
    [request setHTTPMethod:@"GET"];
    
    NSError *error = nil;
    NSURLResponse *response = nil;
    NSData *myConnection = [NSURLConnection sendSynchronousRequest:request returningResponse:&response error:&error];
    NSArray *results = myConnection ? [NSJSONSerialization JSONObjectWithData:myConnection options:0 error:&error] : nil;
    
    if (error) NSLog(@"[%@ %@] JSON error: %@", NSStringFromClass([self class]), NSStringFromSelector(_cmd), error.localizedDescription);
    NSLog(@"[%@ %@] received %@", NSStringFromClass([self class]), NSStringFromSelector(_cmd), results);
    return results;
}

+ (NSArray *)getNewsForCourse:(NSString *)courseId {
    NSString *urlRequest = [[NSString alloc] initWithFormat:@"http://vm-6115.idi.ntnu.no/abc/getNewsForCourse/%@", courseId];
    NSLog(@"URL REQUEST: %@", urlRequest);
    NSMutableURLRequest *request = [NSMutableURLRequest requestWithURL:[NSURL URLWithString:urlRequest]];
    [request addValue:@"application/json; charset=utf-8" forHTTPHeaderField:@"Content-Type"];
    [request setHTTPMethod:@"GET"];
    
    NSError *error = nil;
    NSURLResponse *response = nil;
    NSData *myConnection = [NSURLConnection sendSynchronousRequest:request returningResponse:&response error:&error];
    NSArray *results = myConnection ? [NSJSONSerialization JSONObjectWithData:myConnection options:0 error:&error] : nil;
    
    if (error) NSLog(@"[%@ %@] JSON error: %@", NSStringFromClass([self class]), NSStringFromSelector(_cmd), error.localizedDescription);
    NSLog(@"[%@ %@] received %@", NSStringFromClass([self class]), NSStringFromSelector(_cmd), results);
    return results;
}

+ (NSArray *)getAllSchools {
    NSString *urlRequest = [[NSString alloc] initWithString:@"http://vm-6115.idi.ntnu.no:8080/abc/schools"];
    NSMutableURLRequest *request = [NSMutableURLRequest requestWithURL:[NSURL URLWithString:urlRequest]];
    [request setHTTPMethod:@"GET"];
    
    NSError *error = nil;
    NSURLResponse *response = nil;
    NSData *myConnection = [NSURLConnection sendSynchronousRequest:request returningResponse:&response error:&error];
    NSArray *results = myConnection ? [NSJSONSerialization JSONObjectWithData:myConnection options:0 error:&error] : nil;
    
    if (error) NSLog(@"[%@ %@] JSON error: %@", NSStringFromClass([self class]), NSStringFromSelector(_cmd), error.localizedDescription);
    NSLog(@"[%@ %@] received %@", NSStringFromClass([self class]), NSStringFromSelector(_cmd), results);
    return results;
}

@end














