package no.ntnu.jf.controller;

import java.util.List;

import javax.servlet.http.HttpServletRequest;

import no.ntnu.jf.model.Admin;
import no.ntnu.jf.model.AdminInterface;
import no.ntnu.jf.model.Course;
import no.ntnu.jf.model.School;
import no.ntnu.jf.service.AdminService;
import no.ntnu.jf.service.CourseService;
import no.ntnu.jf.service.SchoolService;
import no.ntnu.jf.service.SpringLoginService;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.servlet.ModelAndView;

@Controller
public class AdminController {
	
	@Autowired
	private AdminService adminService;
	
	@Autowired
	private CourseService courseService;
	
	@Autowired
	private SchoolService schoolService;
	
	private SpringLoginService loginService = new SpringLoginService();
	
	/*
	 *  Spring MVC Methods
	 */
	
	/***
	 * @author Jon Freberg
	 * @param request
	 * @return A ModelAndView object that holds a list of admins and a school object
	 */
	@PreAuthorize("hasRole('ROLE_ADMIN')")
	@RequestMapping(value = "/navigation/admins", method = RequestMethod.GET)
	public ModelAndView listAdmins(HttpServletRequest request) {
		ModelAndView mv = new ModelAndView();
		UserDetails adm = loginService.currentUserDetails();
		Admin admin = (Admin)adminService.getAdminByEmail(adm.getUsername());
		School school = schoolService.getSchoolByShortName(admin.getSchool().getSchoolShortName());
		
		mv.setViewName("admins");
		mv.addObject("adminlist", adminService.findAllAdminsForSchool(school.getSchoolID()));
		mv.addObject("school", school);
		return mv;
	}
	
	
	/*
	 *  CRUD Web Services
	 */

	
	/***
	 * @author Jon Freberg
	 * @param admin (Format: as json object in the HTTP Body)
	 * @return Returns a valid admin object if creation was valid, admin object without ID if not.
	 */
	@RequestMapping(value = "/admin", method = RequestMethod.POST)
	@ResponseBody
	public Admin saveAdmin(@RequestBody Admin admin) {
		adminService.addAdmin(admin);
		return admin;
	}
	
	/***
	 * @author Jon Freberg
	 * @param adminId
	 * @return An admin object. Null if admin with corresponding ID does not exist.
	 */
	@PreAuthorize("hasRole('ROLE_ADMIN')")
	@RequestMapping(value = "/admin/{adminId}", method = RequestMethod.GET)
	@ResponseBody
	public Admin getAdminById(@PathVariable long adminId) {
		Admin admin = adminService.getAdminById(adminId);
		return admin;
	}
	
}
