package no.ntnu.jf.controller;

import java.util.List;

import javax.servlet.http.HttpServletRequest;

import no.ntnu.jf.model.Admin;
import no.ntnu.jf.model.News;
import no.ntnu.jf.service.AdminService;
import no.ntnu.jf.service.CourseService;
import no.ntnu.jf.service.NewsService;
import no.ntnu.jf.service.SchoolService;
import no.ntnu.jf.service.SpringLoginService;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.servlet.ModelAndView;

@Controller
public class NewsController {

	@Autowired
	private NewsService newsService;
	
	@Autowired
	private CourseService courseService;
	
	@Autowired
	private AdminService adminService;
	
	@Autowired
	private SchoolService schoolService;
	
	private SpringLoginService loginService = new SpringLoginService();

	/*
	 * Spring MVC Methods
	 */
	
	/***
	 * @author Jon Freberg
	 * @return A ModelAndView object that holds a list of courses and an admin object.
	 */
	@PreAuthorize("hasRole('ROLE_ADMIN')")
	@RequestMapping(value = "/navigation/news", method = RequestMethod.GET)
	public ModelAndView listNews() {
		ModelAndView mv = new ModelAndView();
		UserDetails adm = loginService.currentUserDetails();
		Admin admin = (Admin)adminService.getAdminByEmail(adm.getUsername());
		
		mv.setViewName("news");
		mv.addObject("courselist", courseService.findAllCoursesForAdmin(admin.getId()));
		mv.addObject("admin", admin);
		
		return mv;
	}
	
	/***
	 * @author Jon Freberg
	 * @param request
	 * @return A ModelAndView object that holds a list of courses and an admin object
	 */
	@PreAuthorize("hasRole('ROLE_ADMIN')")
	@RequestMapping(value = "/navigation/addOrEditNews", method = RequestMethod.GET)
	public ModelAndView listCourses(HttpServletRequest request) {
		ModelAndView mv = new ModelAndView();
		UserDetails adm = loginService.currentUserDetails();
		Admin admin = (Admin)adminService.getAdminByEmail(adm.getUsername());
		
		mv.setViewName("addOrEditNews");
		mv.addObject("courselist", courseService.findAllCoursesForAdmin(admin.getId()));
		mv.addObject("admin", admin);
		
		return mv;
	}
	
	/*
	 * CRUD Web Services
	 */
	
	/***
	 * @author Jon Freberg
	 * @param news (Format: as json object in the HTTP Body)
	 * @return A valid news object if creation was OK. A news object with ID = 0 if an error occurred.
	 */
	@PreAuthorize("hasRole('ROLE_ADMIN')")
	@RequestMapping(value = "/news", method = RequestMethod.POST)
	@ResponseBody
	public News saveNews(@RequestBody News news) {
		newsService.saveNews(news);
		return news;
	}
	
	/***
	 * @author Jon Freberg
	 * @param courseId
	 * @return A list of news objects that belongs to the course with the corresponding ID given as parameter. 
	 */
	@RequestMapping(value = "/getNewsForCourse/{courseId}", method = RequestMethod.GET)
	@ResponseBody
	public List<News> getNewsForCourse(@PathVariable long courseId) {
		List<News> newsList = newsService.findAllNewsForCourse(courseId);
		return newsList;
	}
	
}
