package no.ntnu.jf.dao;

import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.TypedQuery;

import no.ntnu.jf.model.Admin;
import no.ntnu.jf.model.AdminInterface;
import no.ntnu.jf.model.School;
import no.ntnu.jf.service.QueryNames;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;
/***
 * 
 * @author Jon Freberg
 * 
 * Implementation of the AdminDAO object. 
 * Information about the different annotations used and this classes functionality is given in chapter 5.3.2 in the report.
 * 
 */
@Repository
public class AdminDAOImpl implements AdminDAO {

	@PersistenceContext
	EntityManager entityManager;
	
	@Autowired
	private SchoolDAOImpl schoolRepository;
	
	public void setEntityManager(EntityManager entityManager) {
		this.entityManager = entityManager;
	}

	public EntityManager getEntityManager(){
		return entityManager;
	}
	
	public Admin saveAdmin(Admin a) {
		Admin ad = (Admin)getAdminByEmail(a.getEmail());
		
		if(ad == null) {
			System.out.println("SKAL TIL  OPPRETTE ADMIN");
			Admin admin = new Admin();
			admin.setFirstName(a.getFirstName());
			admin.setLastName(a.getLastName());
			admin.setEmail(a.getEmail());
			admin.setPassword(a.getPassword());
			School school = schoolRepository.getSchoolByShortName(a.getSchool().getSchoolShortName());
			System.out.println("School shortname: " + school.getSchoolShortName());
			admin.setSchool(school);
			entityManager.persist(admin);
			return admin;
		}
		return null;
	}

	public void removeAdmin(Admin a) {
		entityManager.remove(a);
	}

	public List<AdminInterface> findAllAdmins() {
		TypedQuery<AdminInterface> query = entityManager.createNamedQuery(QueryNames.FIND_ALL_ADMINS, AdminInterface.class);
		return (List<AdminInterface>) query.getResultList();
	}
	
	public List<Admin> findAllAdminsForSchool(long schoolId) {
		TypedQuery<Admin> query = entityManager.createNamedQuery(QueryNames.FIND_ALL_ADMIN_FOR_SCHOOL, Admin.class);
		query.setParameter(1, schoolId);
		
		try {
			List<Admin> adminsForSchoolList = query.getResultList();
			return adminsForSchoolList;
		} catch (Exception e) {
			System.err.println("Couldnt find any admins for this school");
			return null;
		}
	}

	public AdminInterface getAdminByEmail(String email) {
		TypedQuery<AdminInterface> query = entityManager.createNamedQuery(QueryNames.FIND_ADMIN_BY_EMAIL, AdminInterface.class);
		query.setParameter(1, email);
		try {
			AdminInterface admin = query.getSingleResult();
			return admin;
		} catch (Exception e) {
			System.err.println("The admin with email " + email + " doesnt exist.");
			return null;
		}
		
	}

	public Admin getAdminById(long id) {
		TypedQuery<Admin> query = entityManager.createNamedQuery(QueryNames.FIND_ADMIN_BY_ID, Admin.class);
		query.setParameter(1, id);
		return query.getSingleResult();
	}
}
