package no.ntnu.jf.dao;

import java.util.ArrayList;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.TypedQuery;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import no.ntnu.jf.model.Admin;
import no.ntnu.jf.model.Course;
import no.ntnu.jf.model.School;
import no.ntnu.jf.model.User;
import no.ntnu.jf.service.QueryNames;
/***
 * 
 * @author Jon Freberg
 * 
 * Implementation of the CourseDAO object. 
 * Information about the different annotations used and this classes functionality is given in chapter 5.3.2 in the report.
 * 
 */
@Repository
public class CourseDAOImpl implements CourseDAO {

	@PersistenceContext
	EntityManager entityManager;
	
	@Autowired
	private SchoolDAOImpl schoolRepository;
	
	@Autowired
	private UserDAOImpl userRepository;
	
	@Autowired
	private AdminDAOImpl adminRepository;
	
	public void setEntityManager(EntityManager entityManager) {
		this.entityManager = entityManager;
	}
	
	public EntityManager getEntityManager() {
		return entityManager;
	}
	
	public Course saveCourse(Course c) {
		System.out.println("saveCourse er kalt.");
		
		//Se om course finnes
		Course courseTemp = findCourseByCourseCode(c.getCourseCode());
		
//		List<User> tempUserList = new ArrayList<User>();
//		for(User user : c.getUsers()) {
//			User tempUser = (User)userRepository.getUserByEmail(user.getEmail());
//			tempUserList.add(tempUser);
//		}
//
//		c.setUsers(tempUserList);
		
		if(courseTemp == null) {
			System.err.println("COURSE FANTES IKKE");
			try {
				if(c.getCourseId() > 0) {
					return entityManager.merge(c);
				} else {
					if(c.getSchool() == null) {
	//					School school = schoolRepository.getSchoolById(c.getSchool().getSchoolID());
						School school = schoolRepository.getSchoolByShortName(c.getSchool().getSchoolShortName());
						c.setSchool(school);
					}
					
					if(c.getAdmin() == null) {
	//					Admin admin = adminRepository.getAdminById(c.getAdmin().getId());
						Admin admin = (Admin)adminRepository.getAdminByEmail(c.getAdmin().getEmail());
						c.setAdmin(admin);
					}
					entityManager.persist(c);
					return c;
				}
			} catch (Exception e) {
				System.err.println("Something went wrong when trying to save course.");
				return null;
			}
		} else {
			System.err.println("saveCourse returnerer NULL");
			return null;
		}
	}
	
	// TODO: OPPDATER USER SIN LISTE MED COURSES OGS!
	public Course addUsersToCourse(int[] userIds, long courseId) {
		try {
			Course course = this.findCourseById(courseId);
			
			List<User> tempUserList = new ArrayList<User>();
			for (User user : course.getUsers()) {
				User tempUser = userRepository.getUserById(user.getId());
				tempUserList.add(tempUser);
			}
			
			course.setUsers(tempUserList);
	
			List<User> userList = new ArrayList<User>();
			
			for(int i = 0; i < userIds.length; i++) {
				userList.add(userRepository.getUserById(userIds[i]));
			}
	//		course.setUsers(userList);
			
//			if(course != null) {
				if(userList.size() < 1) {
					System.out.println("USERLIST HAR LENGDE 0");
				}
				for(User user : userList) {
					System.out.println("User FIRSTNAME: " + user.getFirstName());
					course.setUser(user);
					user.addCourse(course);
				}
				entityManager.merge(course);
				return course;
//			}
//			return course;
		} catch(Exception e) {
			System.err.println("Couldnt add users to course: " + e);
			return null;
		}
		
	}

	public boolean removeCourse(Course c) {
		try {
			c.setSchool(new School());
//			c.setUser(new ArrayList<User>());
			entityManager.merge(c);
			entityManager.remove(c);
			return true;
		} catch (Exception e) {
			System.err.println("Something went wrong when trying to delete a course");
			e.printStackTrace();
			return false;
		}
	}

	public List<Course> findAllCourses() {
		TypedQuery<Course> query = entityManager.createNamedQuery(QueryNames.FIND_ALL_COURSES, Course.class);
		return query.getResultList();
	}
	
	public Course findCourseById(long id) {
		TypedQuery<Course> query = entityManager.createNamedQuery(QueryNames.FIND_COURSE_BY_ID, Course.class);
		query.setParameter(1, id);
		return query.getSingleResult();
	}
	
	public Course findCourseByCourseCode(String courseCode) {
		TypedQuery<Course> query = entityManager.createNamedQuery(QueryNames.FIND_COURSE_BY_COURSE_CODE, Course.class);
		query.setParameter(1, courseCode);
		
		try {
			Course c = query.getSingleResult();
			return c;
		} catch (Exception e) {
			System.err.println("Course with course code" + courseCode + " wasnt found.");
			return null;
		}
	}
	
	public List<Course> findAllCoursesForAdmin(long adminId) {
		TypedQuery<Course> query = entityManager.createNamedQuery(QueryNames.FIND_ALL_COURSES_FOR_ADMIN, Course.class);
		query.setParameter(1, adminId);
		
		try {
			List<Course> courseListForAdmin = query.getResultList();
			return courseListForAdmin;
		} catch(Exception e) {
			System.err.println("Admin did not have any courses.");
			return null;
		}
	}

	public List<Course> findAllCoursesForSchool(String shortName) {
		TypedQuery<Course> query = entityManager.createNamedQuery(QueryNames.FIND_ALL_COURSES_FOR_SCHOOL, Course.class);
		query.setParameter(1, shortName);
		return query.getResultList();
	}
	
	// java.lang.IllegalArgumentException: NamedQuery of name: findSchoolByID not found
	public School findSchoolForCourse(long courseId) {
		TypedQuery<Course> query = entityManager.createNamedQuery(QueryNames.FIND_COURSE_BY_ID, Course.class);
		query.setParameter(1, courseId);
		
		Course c = query.getSingleResult();
//		Admin a = adminRepository.getAdminById(c.getAdmin().getId());
		//schoolRepository.getSchoolById(c.getSchool().getSchoolID());
		School s = c.getSchool();
		return s;
	}

	// TODO: HA DISSE I USER KLASSEN??
	public List<User> findAllUsersNotInCourseForSchool(long courseId, long schoolId) {
		TypedQuery<User> query = entityManager.createNamedQuery(QueryNames.FIND_ALL_USERS_BY_SCHOOL, User.class);
		query.setParameter(1, schoolId);
		
		List<User> fullUserList = query.getResultList();
		List<User> usersNotInCourse = new ArrayList<User>();
		boolean userTakesCourse = false;
		
		for(User user : fullUserList) {
			for(Course course : user.getCourseList()) {
				if(course.getCourseId() == courseId) {
					userTakesCourse = true;
				}
			}
			if(!userTakesCourse) {
				System.out.println("La til " + user.getFirstName() + ".");
				usersNotInCourse.add(user);
			}
			userTakesCourse = false;
		}
		return usersNotInCourse;
	}

	public List<User> findAllUsersInCourseForSchool(long courseId, long schoolId) {
		TypedQuery<User> query = entityManager.createNamedQuery(QueryNames.FIND_ALL_USERS_BY_SCHOOL, User.class);
		query.setParameter(1, schoolId);
		
		List<User> fullUserList = query.getResultList();
		List<User> usersInCourse = new ArrayList<User>();
		boolean userTakesCourse = false;
		
		for(User user : fullUserList) {
			for(Course course : user.getCourseList()) {
				if(course.getCourseId() == courseId) {
					userTakesCourse = true;
				}
			}
			if(userTakesCourse) {
				System.out.println("La til " + user.getFirstName() + ".");
				usersInCourse.add(user);
			}
			userTakesCourse = false;
		}
		return usersInCourse;
	}

//	public List<Course> findAllCoursesForUser(String email) {
//		User user = UserDAO
//		
//		TypedQuery<Course> query = entityManager.createNamedQuery(QueryNames.FIND_ALL_COURSES_FOR_SCHOOL, Course.class);
//		
//		
//		return null;
//	}
}



