package no.ntnu.jf.dao;

import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.TypedQuery;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import no.ntnu.jf.model.News;
import no.ntnu.jf.model.School;
import no.ntnu.jf.model.User;
import no.ntnu.jf.service.QueryNames;
/***
 * 
 * @author Jon Freberg
 * 
 * Implementation of the NewsDAO object. 
 * Information about the different annotations used and this classes functionality is given in chapter 5.3.2 in the report.
 * 
 */
@Repository
public class NewsDAOImpl implements NewsDAO {

	@PersistenceContext
	EntityManager entityManager;
	
	@Autowired
	private SchoolDAOImpl schoolRepository;	// TODO: GJR OM TIL COURSE!
	
	public void setEntityManager(EntityManager entityManager) {
		this.entityManager = entityManager;
	}
	
	public EntityManager getEntityManager() {
		return entityManager;
	}
	
	// TODO: Bde persist ved ny nyhet, og merge ved oppdatering av nyhet!
	public News saveNews(News n) {
		
		// Find news by text and courseId i stedet.
//		News news = findNewsById(n.getNewsId());
		News news = findNewsByTextAndCourseId(n.getText(), n.getCourse().getCourseId());
//		System.out.println("COURSE ID: " + );
		
		try {
			if(news == null) {
				entityManager.persist(n);
				return n;
			} else {
				//entityManager.merge(n);
				return null;
			}
		} catch (Exception e) {
			System.err.println("Something went wrong when trying to save/update news");
			e.printStackTrace();
			return null;
		}
	}
	
	public News findNewsById(long newsId) {
		TypedQuery<News> query = entityManager.createNamedQuery(QueryNames.FIND_NEWS_BY_ID, News.class);
		query.setParameter(1, newsId);
		
		try {
			News news = query.getSingleResult();
			return news;
		} catch (Exception e) {
			System.err.println("No news with that ID. Returning NULL for creation of a new News object");
			return null;
		} 
	}
	
	public News findNewsByTextAndCourseId(String text, long courseId) {
		TypedQuery<News> query = entityManager.createNamedQuery(QueryNames.FIND_NEWS_BY_TEXT_AND_COURSEID, News.class);
		query.setParameter(1, text);
		query.setParameter(2, courseId);
		
		try {
			News news = query.getSingleResult();
			return news;
		} catch(Exception e) {
			System.err.println("No news found with this text and courseID. Return NULL for creation of a new News object");
			return null;
		}	
	}

	public List<News> findAllNews() {
		TypedQuery<News> query = entityManager.createNamedQuery(QueryNames.FIND_ALL_NEWS, News.class);
		return query.getResultList();
	}

	public List<News> findAllNewsForCourse(long courseId) {
		TypedQuery<News> query = entityManager.createNamedQuery(QueryNames.FIND_ALL_NEWS_FOR_COURSE, News.class);
		query.setParameter(1, courseId);
		
		try {
			List<News> newsListForCourse = query.getResultList();
			return newsListForCourse;
		} catch (Exception e) {
			System.err.println("No news found for the course with id " + courseId);
			e.printStackTrace();
			return null;
		}
	}
	
	public boolean removeNews(News n) {
		try {
			entityManager.remove(n);
			return true;
		} catch (Exception e) {
			System.err.println("Something went wrong when trying to delete news");
			e.printStackTrace();
		}
		return false;
	}
}
