package no.ntnu.jf.dao;

import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.TypedQuery;

import org.springframework.stereotype.Repository;

import no.ntnu.jf.model.School;
import no.ntnu.jf.service.QueryNames;
/***
 * 
 * @author Jon Freberg
 * 
 * Implementation of the SchoolDAO object. 
 * Information about the different annotations used and this classes functionality is given in chapter 5.3.2 in the report.
 * 
 */
@Repository
public class SchoolDAOImpl implements SchoolDAO {

	@PersistenceContext
	EntityManager entityManager;
	
	public void setEntityManager(EntityManager entityManager) {
		this.entityManager = entityManager;
	}
	
	public EntityManager getEntityManager() {
		return entityManager;
	}
	
	public School saveSchool(School s) {
		if (s.getSchoolID() > 0) {
			entityManager.merge(s);
		} else {
			entityManager.persist(s);
		}
		return s;
	}

	public boolean removeSchool(School s) {
		try {
			entityManager.remove(s);
			return true;
		} catch (Exception e) {
			e.printStackTrace();
			return false;
		}
	}

	public List<School> findAllSchools() {
		TypedQuery<School> query = entityManager.createNamedQuery(QueryNames.FIND_ALL_SCHOOLS, School.class);
		return query.getResultList();
	}

	public School getSchoolByShortName(String shortName) {
		TypedQuery<School> query = entityManager.createNamedQuery(QueryNames.FIND_SCHOOL_BY_NAME, School.class);
		query.setParameter(1, shortName);
		return query.getSingleResult();
	}

	public School getSchoolByFullName(String fullName) {
		System.out.println("Not implemented");
		return null;
	}

	public School getSchoolById(long id) {
		TypedQuery<School> query = entityManager.createNamedQuery(QueryNames.FIND_SCHOOL_BY_ID, School.class);
		query.setParameter(1, id);
		return query.getSingleResult();
	}

}
