package no.ntnu.jf.dao;

import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.TypedQuery;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import no.ntnu.jf.model.Course;
import no.ntnu.jf.model.School;
import no.ntnu.jf.model.User;
import no.ntnu.jf.model.UserInterface;
import no.ntnu.jf.service.QueryNames;
/***
 * 
 * @author Jon Freberg
 * 
 * Implementation of the UserDAO object. 
 * Information about the different annotations used and this classes functionality is given in chapter 5.3.2 in the report.
 * 
 */
@Repository
public class UserDAOImpl implements UserDAO {
	
	@PersistenceContext
	EntityManager entityManager;
	
	@Autowired
	private SchoolDAOImpl schoolRepository;

	
	public void setEntityManager(EntityManager entityManager) {
		this.entityManager = entityManager;
	}

	public EntityManager getEntityManager(){
		return entityManager;
	}
	
	public User saveUser(User u) {
		
		User usr = (User)getUserByEmail(u.getEmail());
		
		if(usr == null) {
			School school = schoolRepository.getSchoolByShortName(u.getSchool().getSchoolShortName());
			u.setSchool(school);
			entityManager.persist(u);
			return u;
		}
		return null;

	}
	
	public List<User> findAllUsersAtSchool(long schoolId) {
		TypedQuery<User> query = entityManager.createNamedQuery(QueryNames.FIND_ALL_USERS_BY_SCHOOL, User.class);
		query.setParameter(1, schoolId);
		
		try {
			return (List<User>)query.getResultList();
		} catch (Exception e) {
			System.err.println("No users from that school exists.");
			return null;
		}
	}

	public void removerUser(User u) {
		entityManager.remove(u);
	}
	
	public UserInterface getUserByEmail(String email) {
		System.out.println("EMAIL PARAMETER: " + email);
		TypedQuery<UserInterface> query = entityManager.createNamedQuery(QueryNames.FIND_USER_BY_EMAIL, UserInterface.class);
		query.setParameter(1, email);
		System.out.println("EMAIL : " + email);
		try {
        	UserInterface user = query.getSingleResult();
        	System.out.println("USER FOUND");
        	System.out.println(user.getId());
        	System.out.println(user.getEmail());
        	System.out.println(user.getPassword());
            return user;
        } catch (Exception e) {
        	System.err.println("The user with email " + email + " doesn't exist.");
            return null;
        }
	}

	public List<UserInterface> findAllUsers() {
		TypedQuery<UserInterface> query = entityManager.createNamedQuery(QueryNames.FIND_ALL_USERS, UserInterface.class);
		return (List<UserInterface>) query.getResultList();
	}
	
	public List<Course> getAllCoursesForUser() {
		return null;
	}

	public User getUserById(long id) {
		TypedQuery<User> query = entityManager.createNamedQuery(QueryNames.FIND_USER_BY_ID, User.class);
		query.setParameter(1, id);
		return query.getSingleResult();
	}

}
