package no.ntnu.jf.model;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.Table;
import javax.persistence.UniqueConstraint;

import no.ntnu.jf.service.Queries;
import no.ntnu.jf.service.QueryNames;

import org.codehaus.jackson.annotate.JsonIgnore;
import org.codehaus.jackson.map.annotate.JsonDeserialize;

/***
 * 
 * @author Jon Freberg
 * 
 * Implementation of the course model object. Information about the different annotations used is given in chapter 5.3.1 in the report.
 * 
 */
@Entity
@Table(name = "t_course", uniqueConstraints = @UniqueConstraint(columnNames = { "course_code" }))
@NamedQueries({
		@NamedQuery(name = QueryNames.FIND_ALL_COURSES, query = Queries.FIND_ALL_COURSES_QUERY),
		@NamedQuery(name = QueryNames.FIND_ALL_COURSES_FOR_SCHOOL, query = Queries.FIND_ALL_COURSES_FOR_SCHOOL_QUERY),
		@NamedQuery(name = QueryNames.FIND_ALL_COURSES_FOR_ADMIN, query = Queries.FIND_ALL_COURSES_FOR_ADMIN_QUERY),
		@NamedQuery(name = QueryNames.FIND_COURSE_BY_ID, query = Queries.FIND_COURSE_BY_ID_QUERY),
		@NamedQuery(name = QueryNames.FIND_COURSE_BY_COURSE_CODE, query = Queries.FIND_COURSE_BY_COURSE_CODE_QUERY),
		@NamedQuery(name = QueryNames.FIND_SCHOOL_FOR_COURSE, query = Queries.FIND_SCHOOL_FOR_COURSE_QUERY) })
public class Course implements CourseInterface, Serializable {

	private static final long serialVersionUID = 1L;

	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	@Column(name = "course_id")
	public long courseId;

	@Column(name = "course_code", nullable = false)
	public String courseCode;

	@Column(name = "course_name", nullable = false)
	public String courseName;

	@Column(name = "course_description", columnDefinition = "TEXT", nullable = false)
	public String courseDescription;

	@ManyToOne(cascade = { CascadeType.MERGE })
	@JoinColumn(name = "school_id", nullable = false)
	private School school;

	@ManyToOne(cascade = { CascadeType.MERGE })
	@JoinColumn(name = "admin_id", nullable = false)
	private Admin admin;

	@ManyToMany(cascade = { CascadeType.MERGE }, fetch = FetchType.EAGER)
	@JoinTable(name = "t_user_course", joinColumns = { @JoinColumn(name = "course_id", referencedColumnName = "course_id") }, inverseJoinColumns = { @JoinColumn(name = "user_id", referencedColumnName = "user_id") })
	private List<User> users;

	public Course() {
	}

	public long getCourseId() {
		return courseId;
	}

	public void setCourseId(long courseId) {
		this.courseId = courseId;
	}

	public School getSchool() {
		return school;
	}

	@JsonDeserialize(as = School.class)
	public void setSchool(School school) {
		this.school = school;
	}

	public Admin getAdmin() {
		return admin;
	}

	@JsonDeserialize(as = Admin.class)
	public void setAdmin(Admin admin) {
		this.admin = admin;
	}

	public String getCourseCode() {
		return this.courseCode;
	}

	public void setCourseCode(String courseCode) {
		this.courseCode = courseCode;
	}

	public String getCourseName() {
		return courseName;
	}

	public void setCourseName(String courseName) {
		this.courseName = courseName;
	}

	// @JsonIgnore
	@JsonIgnore
	public List<User> getUsers() {
		return users;
	}

	// @JsonDeserialize(as = ArrayList.class, contentAs = User.class)
	// public List<User> getUsersJSON() {
	// return users;
	// }

	// @JsonDeserialize(as = ArrayList.class, contentAs = User.class)
	@JsonIgnore
	public void setUsers(List<User> users) {
		this.users = users;
	}

	// @JsonDeserialize(as = User.class)
	@JsonIgnore
	public void setUser(User user) {
		this.users.add(user);
	}

	// public void addUser(User user) {
	// users.add(user);
	// }

	// public User getUser() {
	// // TODO Auto-generated method stub
	// return null;
	// }
	//
	//
	// public void setUser(User user) {
	//
	//
	// }

	// public void addQuizQuestion(QuizQuestion quizQuestion) {
	// questionList.add((QuizQuestionDO)quizQuestion);
	// quizQuestion.setQuiz(this);
	// }

}
