package no.ntnu.jf.model;

import java.io.Serializable;
import java.util.Date;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.UniqueConstraint;

import org.codehaus.jackson.map.annotate.JsonDeserialize;

import no.ntnu.jf.service.Queries;
import no.ntnu.jf.service.QueryNames;
/***
 * 
 * @author Jon Freberg
 * 
 * Implementation of the news model object. Information about the different annotations used is given in chapter 5.3.1 in the report.
 * 
 */
@Entity
@Table(name = "t_news", uniqueConstraints = @UniqueConstraint(columnNames={"topic"}))
	@NamedQueries({
		@NamedQuery(name = QueryNames.FIND_ALL_NEWS, query = Queries.FIND_ALL_NEWS_QUERY),
		@NamedQuery(name = QueryNames.FIND_NEWS_BY_ID, query = Queries.FIND_NEWS_BY_ID_QUERY),
		@NamedQuery(name = QueryNames.FIND_NEWS_BY_TEXT_AND_COURSEID, query = Queries.FIND_NEWS_BY_TEXT_AND_COURSEID_QUERY),
		@NamedQuery(name = QueryNames.FIND_ALL_NEWS_FOR_COURSE, query = Queries.FIND_ALL_NEWS_FOR_COURSE_QUERY)
	})
public class News implements NewsInterface, Serializable {

	private static final long serialVersionUID = 1L;
	
	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	@Column(name = "news_id")
	private long newsId;
	
	@JoinColumn(name = "admin_id", nullable = false)
	private Admin admin;
	
	@Temporal(TemporalType.DATE)
	@Column(name = "reg_time")
	private Date regTime;
	
	@Temporal(TemporalType.DATE)
	@Column(name = "mod_time")
	private Date modTime;
	
	// , nullable = false, unique = true
	//columnDefinition = "TEXT",
	// TODO: denne var @NotEmpty som er en Hibernation annotasjon. Satte nullable = false i stedet.
	@Column(name = "newsText", columnDefinition = "TEXT", nullable = false)
	private String text;
	
	@Column(name = "topic", nullable = false)
	private String topic;
	
	@ManyToOne(cascade= {CascadeType.MERGE})
	@JoinColumn(name = "course_id", nullable = false)
	private Course course;
	
//	@ManyToOne(cascade= {CascadeType.MERGE})
//	@JoinColumn(name="course_id", nullable=false)
//	private Course course;
	
	public News(){}

	public long getNewsId() {
		return newsId;
	}

	public void setNewsId(long newsId) {
		this.newsId = newsId;
	}

	public Admin getAdmin() {
		return admin;
	}

	@JsonDeserialize(as = Admin.class)
	public void setAdmin(Admin admin) {
		this.admin = admin;
	}
	
	public Course getCourse() {
		return course;
	}
	
	@JsonDeserialize(as = Course.class)
	public void setCourse(Course course) {
		this.course = course;
	}

	public Date getModTime() {
		return modTime;
	}

	public void setModTime(Date modTime) {
		this.modTime = modTime;
	}

	public Date getRegTime() {
		return regTime;
	}

	public void setRegTime(Date regTime) {
		this.regTime = regTime;
	}

	public String getText() {
		return text;
	}

	public void setText(String text) {
		this.text = text;
	}

	public String getTopic() {
		return topic;
	}

	public void setTopic(String topic) {
		this.topic = topic;
	}

}
