package no.ntnu.jf.model;

import java.io.Serializable;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.Table;
import javax.persistence.UniqueConstraint;

import no.ntnu.jf.service.QueryNames;
import no.ntnu.jf.service.Queries;
/***
 * 
 * @author Jon Freberg
 * 
 * Implementation of the school model object. Information about the different annotations used is given in chapter 5.3.1 in the report.
 * 
 */
@Entity
@Table(name="t_school", uniqueConstraints=@UniqueConstraint(columnNames={"school_name"}))
@NamedQueries({
    @NamedQuery(name = QueryNames.FIND_ALL_SCHOOLS, query = Queries.FIND_ALL_SCHOOLS_QUERY),
    @NamedQuery(name = QueryNames.FIND_SCHOOL_BY_NAME, query = Queries.FIND_SCHOOL_BY_NAME_QUERY)
})
public class School implements Serializable, SchoolInterface {
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	@Column(name="school_id", nullable=false)
	private long id; 
	
	@Column(name="school_shortname", nullable=false)
	private String shortName; 
	
	@Column(name="school_name", nullable=false)
	private String fullName;
	
	@Column(name="school_active", nullable=false)
	private boolean active;
	
	public School(){}

	public long getSchoolID() {
		return id;
	}

	public void setSchoolID(long id) {
		this.id = id;
	}

	public String getSchoolShortName() {
		return shortName;
	}

	public void setSchoolShortName(String schoolShortName) {
		shortName = schoolShortName;
	}

	public String getSchoolFullName() {
		return fullName;
	}

	public void setSchoolFullName(String schoolFullName) {
		fullName = schoolFullName;
	}

	public boolean getActive() {
		return active;
	}

	public void setActive(boolean active) {
		this.active = active;
	}
	
}
