package no.ntnu.jf.model;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.Table;
import javax.persistence.UniqueConstraint;

import org.codehaus.jackson.annotate.JsonIgnore;
import org.codehaus.jackson.map.annotate.JsonDeserialize;

import no.ntnu.jf.service.*;
/***
 * 
 * @author Jon Freberg
 * 
 * Implementation of the user model object. Information about the different annotations used is given in chapter 5.3.1 in the report.
 * 
 */
@Entity
@Table(name = "t_user", uniqueConstraints = @UniqueConstraint(columnNames={"email"}))
@NamedQueries({
    @NamedQuery(name = QueryNames.FIND_ALL_USERS, query = Queries.FIND_ALL_USERS_QUERY),
    @NamedQuery(name = QueryNames.FIND_USER_BY_EMAIL, query = Queries.FIND_USER_BY_EMAIL_QUERY),
    @NamedQuery(name = QueryNames.FIND_ALL_USERS_BY_SCHOOL, query = Queries.FIND_ALL_USERS_BY_SCHOOL_QUERY),
    @NamedQuery(name = QueryNames.FIND_USER_BY_ID, query = Queries.FIND_USER_BY_ID_QUERY)
})

public class User implements UserInterface, Serializable {
	
	private static final long serialVersionUID = 1L;
	
	@Id
	@GeneratedValue(strategy=GenerationType.IDENTITY)
	@Column(name = "user_id", nullable = false)
	private long id;
	
	@Column(name = "firstname", nullable = false, length = 30)
	private String firstName;
	
	@Column(name = "lastname", nullable = false, length = 30)
	private String lastName;
	
	@Column(name = "email", nullable = false, length = 100)
	private String email;
	
	@ManyToOne(cascade= {CascadeType.MERGE})
	@JoinColumn(name="school_id", nullable=false)
	private School school;
	
//	@ManyToMany(cascade = {CascadeType.MERGE}, fetch = FetchType.EAGER)
//	@JoinTable(
//			name = "t_user_course",
//			joinColumns = {@JoinColumn(name = "user_id", referencedColumnName = "user_id")},
//			inverseJoinColumns = {@JoinColumn(name = "course_id", referencedColumnName = "course_id")})
	@ManyToMany(mappedBy = "users")
	private List<Course> courseList;
	
	@Column(name = "graduation_year", nullable = false)
	private int graduationYear;
	
	@Column(name = "password", nullable = false, length = 40)
	private String password;
	
	public User(){}
	
//	public User(String firstName, String lastName, String email, int graduationYear, String password){
//		this.firstName = firstName;
//		this.lastName = lastName;
//		this.email = email;
//		this.graduationYear = graduationYear;
//		this.password = password;
//	}
//	
//	public User(long id, String firstName, String lastName, String email, int graduationYear, String password){
//		this.id = id;
//		this.firstName = firstName;
//		this.lastName = lastName;
//		this.email = email;
//		this.graduationYear = graduationYear;
//		this.password = password;
//	}

	public long getId() {
		return id;
	}

	public void setId(long id) {
		this.id = id;
	}

	public String getFirstName() {
		return firstName;
	}

	public void setFirstName(String firstName) {
		this.firstName = firstName;
	}

	public String getLastName() {
		return lastName;
	}

	public void setLastName(String lastName) {
		this.lastName = lastName;
	}

	public String getEmail() {
		return email;
	}

	public void setEmail(String email) {
		this.email = email;
	}

	public int getGraduationYear() {
		return graduationYear;
	}

	public void setGraduationYear(int graduationYear) {
		this.graduationYear = graduationYear;
	}

	public String getPassword() {
		return password;
	}

	public void setPassword(String password) {
		this.password = password;
	}

	public School getSchool() {
		return school;
	}

	@JsonDeserialize(as = School.class)
	public void setSchool(School school) {
		this.school = school;
	}

	@JsonIgnore
	public List<Course> getCourseList() {
		return courseList;
	}
	
	@JsonDeserialize(as = ArrayList.class, contentAs = Course.class)
	public List<Course> getCourseListJSON() {
		return courseList;
	}

//	@JsonDeserialize(as = ArrayList.class, contentAs = Course.class)
	@JsonIgnore
	public void setCourseList(List<Course> courseList) {
		this.courseList = courseList;
	}
	
	@JsonIgnore
	public void addCourse(Course course) {
		courseList.add(course);
	}
	
//	public void addQuizQuestion(QuizQuestion quizQuestion) {
//		questionList.add((QuizQuestionDO)quizQuestion);
//		quizQuestion.setQuiz(this);
//	}
}
