package no.ntnu.jf.service;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import no.ntnu.jf.dao.CourseDAO;
import no.ntnu.jf.model.Course;
import no.ntnu.jf.model.School;
import no.ntnu.jf.model.User;
/***
 * 
 * @author Jon Freberg
 * 
 * Implementation of the CourseService object. 
 * Information about the different annotations used and this classes functionality is given in chapter 5.3.3 in the report.
 * 
 */
@Service
public class CourseServiceImpl implements CourseService {

	@Autowired
	private CourseDAO courseDAO;
	
	@Transactional
	public Course saveCourse(Course c) {
		return courseDAO.saveCourse(c);
	}

	@Transactional
	public boolean removeCourse(Course c) {
		return courseDAO.removeCourse(c);
	}
	
	@Transactional
	public Course addUsersToCourse(int[] userIds, long courseId) {
		return courseDAO.addUsersToCourse(userIds, courseId);
	}

	public List<Course> findAllCourses() {
		return courseDAO.findAllCourses();
	}
	
	public Course findCourseById(long id) {
		return courseDAO.findCourseById(id);
	}
	
	public List<Course> findAllCoursesForAdmin(long adminId) {
		return courseDAO.findAllCoursesForAdmin(adminId);
	}

	public List<Course> findAllCoursesForSchool(String shortName) {
		return courseDAO.findAllCoursesForSchool(shortName);
	}

	public School findSchoolForCourse(long courseId) {
		return courseDAO.findSchoolForCourse(courseId);
	}
	
	public List<User> findAllUsersInCourseForSchool(long courseId, long schoolId) {
		return courseDAO.findAllUsersInCourseForSchool(courseId, schoolId);
	}
	
	public List<User> findAllUsersNotInCourseForSchool(long courseId, long schoolId) {
		return courseDAO.findAllUsersNotInCourseForSchool(courseId, schoolId);
	}
}
