package no.ntnu.jf.service;

import java.util.ArrayList;
import java.util.List;

import no.ntnu.jf.model.Admin;
import no.ntnu.jf.model.AdminInterface;
import no.ntnu.jf.service.AdminService;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.GrantedAuthorityImpl;
import org.springframework.security.core.context.SecurityContext;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.User;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

@Service("customUserDetailsService")
@Transactional
public class SpringLoginService implements UserDetailsService {
	
	@Autowired
	AdminService adminService;
	
	/**
	 *	@author jonfr
	 *	@param email The email of the admin that logs in
	 *	@return returns a UserDetail object that represents the admin trying to log in. This method does not use a webservice, but it is
	 *used by the security-context.xml to validate the admin trying to log in.
	 *	
	 */
	
//	@Override
	public UserDetails loadUserByUsername(String email) throws UsernameNotFoundException, DataAccessException {
		UserDetails userDetails = null;
		List<GrantedAuthority> authList = new ArrayList<GrantedAuthority>(1);
		authList.add(new GrantedAuthorityImpl("ROLE_ADMIN"));
		try {
			List<AdminInterface> adminList = checkIfAdminsExist();
			if(adminList.isEmpty()) {
				AdminInterface standardAdmin = (AdminInterface)adminService.getAdminByEmail(email);
				if(standardAdmin != null) {
					userDetails = new User(standardAdmin.getEmail(), standardAdmin.getPassword(), true, true, true, true, authList);					
				}
			} else {
				AdminInterface admin = (AdminInterface)adminService.getAdminByEmail(email);
					userDetails = new User(admin.getEmail(), admin.getPassword(), true, true, true, true, authList);
			}
		} catch(Exception e) {
			e.printStackTrace();
		}
		return userDetails;
	}
	
	public UserDetails currentUserDetails(){
	    SecurityContext securityContext = SecurityContextHolder.getContext();
	    Authentication authentication = securityContext.getAuthentication();
	    if (authentication != null) {
	        Object principal = authentication.getPrincipal();
	        return (UserDetails) (principal instanceof UserDetails ? principal : null);
	    }
	    return null;
	}
	
	private List<AdminInterface> checkIfAdminsExist() {
		List<AdminInterface> adminList = new ArrayList<AdminInterface>();
		try {
			adminList = (List<AdminInterface>) adminService.findAllAdmins();
			return adminList;
		} catch (Exception e) {
			return adminList;
		}
	}

}
