package no.ntnu.jf.service;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import no.ntnu.jf.dao.UserDAO;
import no.ntnu.jf.model.User;
import no.ntnu.jf.model.UserInterface;
/***
 * 
 * @author Jon Freberg
 * 
 * Implementation of the UserService object. 
 * Information about the different annotations used and this classes functionality is given in chapter 5.3.3 in the report.
 * 
 */
@Service
public class UserServiceImpl implements UserService {

	@Autowired
	private UserDAO userDAO;
	
	@Transactional
	public void addUser(User u) {
		userDAO.saveUser(u);
	}

	@Transactional
	public void removeUser(User u) {
		userDAO.removerUser(u);
	}

	public User getUserById(long id) {
		return userDAO.getUserById(id);
	}
	
	public UserInterface getUserByEmail(String email) {
		return userDAO.getUserByEmail(email);
	}
	
	public List<UserInterface> findAllUsers() {
		return userDAO.findAllUsers();
	}
	
	public List<User> findAllUsersAtSchool(long schoolId) {
		return userDAO.findAllUsersAtSchool(schoolId);
	}

	public User tryLogin(User u) {
		User loginUser = (User) userDAO.getUserByEmail(u.getEmail());
		
		if(loginUser != null && loginUser.getPassword().equals(u.getPassword())) {
			return loginUser;
		} else {
			return null;
		}
	}
	
	public User tryLoginWithCreds(String username, String password) {
		User loginUser = (User)userDAO.getUserByEmail(username);
		
		if(loginUser != null && loginUser.getPassword().equals(password)) {
			return loginUser;
		} else {
			return null;
		}
	}

}
